/* 
 *	Copyright (C) Chia-chen Kuo - April 2001
 *
 *  This file is part of DVD2AVI, a free MPEG-2 decoder
 *	
 *  DVD2AVI is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  DVD2AVI is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */

// Added code/menu item to recognize SSE2 processors - trbarry 12/30/2001

#define GLOBAL
#include <math.h>
extern "C"
{
#include "global.h"
#include "AC3Dec\ac3.h"
}
#include "filters.h"

#define TRACK_HEIGHT	30
#define INIT_WIDTH		320
#define INIT_HEIGHT		240

#define MASKCOLOR			RGB(0, 6, 0)
#define MAX_LOADSTRING		100
#define INI_VERSION			1

#define SAVE_AVI		1
#define	SAVE_D2V		2
#define SAVE_WAV		3
#define	SAVE_SESS		4
#define	OPEN_D2V		5
#define OPEN_VOB		6
#define OPEN_WAV		7
#define OPEN_SESS		8

#define PRIORITY_HIGH		1
#define PRIORITY_NORMAL		2
#define PRIORITY_LOW		3

BOOL PopFileDlg(PTSTR, HWND, int);
ATOM MyRegisterClass(HINSTANCE);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK Statistics(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK VideoList(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK AudioList(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK Delay(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK ClipResize(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK Luminance(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK Normalization(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK About(HWND, UINT, WPARAM, LPARAM);
static DWORD DDColorMatch(LPDIRECTDRAWSURFACE, COLORREF);
static void ShowStatistics(bool);
static void DisableControl(int);
static void ClearTrack(void);
static void CheckINI(void);
static void CheckUncheckMenus(void);
static void Recovery(void);
static void RefreshWindow(void);
static void OpenVideoFile(HWND);
static void OpenAudioFile(HWND);
static bool LoadSession(char *szInput, bool preview, bool saveAvi);
static void SaveAvi();
static DWORD WINAPI ProcessWAV(LPVOID n);

static OPENFILENAME ofn, sfn;
static int INI_Version, INIT_X, INIT_Y, Priority_Flag;
static FILE *INIFile, *BMPFile;
static char szPath[_MAX_PATH], szTemp[_MAX_PATH], szWindowClass[MAX_LOADSTRING];
struct _finddata_t vfpfile, seqfile;
static int Quit_Flag = false;
static int AutoQuit_Flag = false;

static HINSTANCE hInst;
static HANDLE hProcess, hThread;
static DWORD threadId;
static HWND hClipResizeDlg, hLumDlg, hNormDlg;

static HDC hdc;
static PAINTSTRUCT ps;
static HBRUSH hBrush;

static char Outfilename[MAX_FILE_NUMBER][_MAX_PATH];
static int SoundDelay[MAX_FILE_NUMBER];

static unsigned char BMPHeader[54] =
{
	0x42, 0x4d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x36, 0x00, 0x00, 0x00, 0x28, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x01, 0x00, 0x18, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x12, 0x0b,
	0x00, 0x00, 0x12, 0x0b, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
	HACCEL hAccel;

	// Initialize global strings
	LoadString(hInstance, IDC_GUI, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	// Perform application initialization
	hInst = hInstance;

	hWnd = CreateWindow(szWindowClass, "DVD2AVI", WS_OVERLAPPEDWINDOW & ~(WS_THICKFRAME|WS_MAXIMIZEBOX),
		CW_USEDEFAULT, 0, INIT_WIDTH, INIT_HEIGHT, NULL, NULL, hInstance, NULL);

	CheckINI();
	memcopy_init();

	hTrack = CreateWindow(TRACKBAR_CLASS, NULL,
		WS_CHILD | WS_VISIBLE | WS_DISABLED | TBS_ENABLESELRANGE | TBS_NOTICKS | TBS_TOP,
		0, INIT_HEIGHT, INIT_WIDTH-4*TRACK_HEIGHT, TRACK_HEIGHT, hWnd, (HMENU) ID_TRACKBAR, hInst, NULL);
	SendMessage(hTrack, TBM_SETRANGE, (WPARAM) true, (LPARAM) MAKELONG(0, TRACK_PITCH));

	hLeftButton = CreateWindow("BUTTON", "[",
		WS_CHILD | WS_VISIBLE | WS_DLGFRAME | WS_DISABLED,
		INIT_WIDTH-4*TRACK_HEIGHT, INIT_HEIGHT,
		TRACK_HEIGHT, TRACK_HEIGHT, hWnd, (HMENU) ID_LEFT_BUTTON, hInst, NULL);

	hLeftArrow = CreateWindow("BUTTON", "<",
		WS_CHILD | WS_VISIBLE | WS_DLGFRAME | WS_DISABLED,
		INIT_WIDTH-3*TRACK_HEIGHT, INIT_HEIGHT,
		TRACK_HEIGHT, TRACK_HEIGHT, hWnd, (HMENU) ID_LEFT_ARROW, hInst, NULL);

	hRightArrow = CreateWindow("BUTTON", ">",
		WS_CHILD | WS_VISIBLE | WS_DLGFRAME | WS_DISABLED,
		INIT_WIDTH-2*TRACK_HEIGHT, INIT_HEIGHT,
		TRACK_HEIGHT, TRACK_HEIGHT, hWnd, (HMENU) ID_RIGHT_ARROW, hInst, NULL);

	hRightButton = CreateWindow("BUTTON", "]",
		WS_CHILD | WS_VISIBLE | WS_DLGFRAME | WS_DISABLED,
		INIT_WIDTH-TRACK_HEIGHT, INIT_HEIGHT,
		TRACK_HEIGHT, TRACK_HEIGHT, hWnd, (HMENU) ID_RIGHT_BUTTON, hInst, NULL);

	ResizeWindow(INIT_WIDTH, INIT_HEIGHT);
	MoveWindow(hWnd, INIT_X, INIT_Y, INIT_WIDTH+Edge_Width, INIT_HEIGHT+Edge_Height+TRACK_HEIGHT, true);
	ShowWindow(hWnd, nCmdShow);

	CheckUncheckMenus();

	// Process command line
	if (strlen(lpCmdLine))
	{
		char command[1024]="";
		char session[1024]="";
		if (sscanf(lpCmdLine, "%s %s", command, session) == 2)
		{
			if (!strcmp(command, "/preview"))
			{
				LoadSession(session, true, false);
			}
			else if (!strcmp(command, "/saveavi"))
			{
				LoadSession(session, false, true);
			}
		}
	}

	// Load accelerators
	hAccel = LoadAccelerators(hInstance, (LPCTSTR)IDR_ACCELERATOR);

	// Main message loop
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(hWnd, hAccel, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return msg.wParam;
}

bool scanOption(FILE* fd, char* name, int count, void* value0, void* value1 = 0, void* value2 = 0, void* value3 = 0, void* value4 = 0, void* value5 = 0)
{
	char line[1024+1];
	
	rewind(fd);
	while (fgets(line, sizeof(line)-1, fd) != NULL)
	{
		if (line[strlen(line)-1] == '\n')
			line[strlen(line)-1] = '\0';
		if (strstr(line, name))
		{
			char *pt = strchr(line, '=');
			if (pt)
			{
				switch(count)
				{
				case 1:
					sscanf(pt+1, "%d", value0);
					break;
				case 2:
					sscanf(pt+1, "%d,%d", value0, value1);
					break;
				case 3:
					sscanf(pt+1, "%d,%X,%X", value0, value1, value2);
					break;
				case 4:
					sscanf(pt+1, "%d,%X,%d,%X", value0, value1, value2, value3);
					break;
				case 6:
					sscanf(pt+1, "%d,%d,%d,%d,%d,%d", value0, value1, value2, value3, value4, value5);
					break;
				default:
					return false;
				}

				return true;
			}
		}
	}
	
	return false;
}

static bool LoadSession(char *szInput, bool preview, bool saveAvi)
{
	FILE *fd = fopen(szInput, "r");
	char line[1024+1];
	int i;

	if (saveAvi)
		AutoQuit_Flag = true;

	strcpy(szOutput, szInput);
	char* ext = strrchr(szOutput, '.');
	if (ext!=NULL && !_strnicmp(ext, ".d2s", 4))
		*ext = '\0';

	while (File_Limit)
	{
		File_Limit--;
		_close(Infile[File_Limit]);
	}
	
	if (fd && (fscanf(fd, "%s", line) == 1) && (!strcmp(line, "DVD2AVISessionFile")))
	{
		while (fgets(line, sizeof(line)-1, fd) != NULL)
		{
			if (line[strlen(line)-1] == '\n')
				line[strlen(line)-1] = '\0';
			char *pt = strchr(line, '=');
			if (strstr(line, "AddFile") && pt)
			{
				strcpy(Infilename[File_Limit], pt+1);
				if ((Infile[File_Limit] = _open(Infilename[File_Limit], _O_RDONLY | _O_BINARY | _O_SEQUENTIAL))==-1)
				{
					i = 0;
					while (i<File_Limit)
					{
						_close(Infile[i]);
						i++;
					}	
					
					File_Limit = 0;
					break;
				}
				else
					File_Limit++;
			}
		}
	}
	
	Recovery();
	
	if (fd)
	{
		if (!scanOption(fd, "Location", 4, &process.leftfile, &process.leftlba, &process.rightfile, &process.rightlba))
		{
			process.leftfile = 0;
			process.leftlba = 0;
			process.rightfile = File_Limit - 1;
			process.rightlba = process.length[File_Limit-1]/BUFFER_SIZE;
		}
		process.startfile = process.leftfile;
		process.startloc = process.leftlba * BUFFER_SIZE;
		process.endfile = process.rightfile;
		process.endloc = (process.rightlba - 1) * BUFFER_SIZE;

		if (!scanOption(fd, "iDCT_Algorithm", 1, &iDCT_Flag))
			iDCT_Flag = IDCT_MMX;
		if (!scanOption(fd, "Field_Operation", 1, &FO_Flag))
			FO_Flag = FO_NONE;
		if (!scanOption(fd, "Store_Flag", 1, &Store_Flag))
			Store_Flag = STORE_YUY2;
		if (!scanOption(fd, "YUVRGB_Scale", 1, &Scale_Flag))
			Scale_Flag = true;
		if (!scanOption(fd, "Luminance", 2, &LumGain, &LumOffset))
		{
			LumGain = 128; LumOffset = 0;
		}
		if (!scanOption(fd, "Picture_Size", 6, &Clip_Top, &Clip_Bottom, &Clip_Left, &Clip_Right, &Squeeze_Width, &Squeeze_Height))
		{
			Clip_Left = Clip_Right = Clip_Top = Clip_Bottom = Squeeze_Width = Squeeze_Height = 0;
		}
		if (!scanOption(fd, "Resize_Function", 1, &Resize_Function))
			Resize_Function = FILTER_BILINEAR;
		
		if (!scanOption(fd, "Track_Number", 1, &Track_Flag))
			Track_Flag = TRACK_NONE;
		if (!scanOption(fd, "Channel_Format", 1, &Format_Flag))
			Format_Flag = FORMAT_AUTO;
		if (!scanOption(fd, "AC3", 1, &AC3_Flag))
			AC3_Flag = AUDIO_DECODE;
		if (!scanOption(fd, "DR_Control", 1, &DRC_Flag))
			DRC_Flag = DRC_NORMAL;
		if (!scanOption(fd, "DS_Downmix", 1, &DSDown_Flag))
			DSDown_Flag = false;
		if (!scanOption(fd, "MPA", 1, &MPA_Flag))
			MPA_Flag = AUDIO_DEMUXONE;
		if (!scanOption(fd, "SRC_Precision", 1, &SRC_Flag))
			SRC_Flag = SRC_NONE;
		if (!scanOption(fd, "Norm_Ratio", 1, &Norm_Ratio))
		{
			Norm_Ratio = 100;
			Normalization_Flag = false;
		}
		else
		{
			if (Norm_Ratio > 100)
			{
				Norm_Ratio -= 100;
				Normalization_Flag = true;
			}
			else
				Normalization_Flag = false;
		}
		if (!scanOption(fd, "Process_Priority", 1, &Priority_Flag))
			Priority_Flag = PRIORITY_NORMAL;
		
		KeyOp_Flag = KEY_OFF;
		Aspect_Change = ASPECT_FREE;
		
		fclose(fd);
	}
	
	if (hLumDlg!=NULL)
	{
		DestroyWindow(hLumDlg);
		hLumDlg = CreateDialog(hInst, (LPCTSTR)IDD_LUMINANCE, hWnd, (DLGPROC)Luminance);
	}
	
	if (hClipResizeDlg!=NULL)
	{
		DestroyWindow(hClipResizeDlg);
		hClipResizeDlg = CreateDialog(hInst, (LPCTSTR)IDD_CLIPRESIZE, hWnd, (DLGPROC)ClipResize);
	}
	
	CheckUncheckMenus();
	
	if (File_Limit)
	{
		process.run = 0;
		for (i=0; i<process.startfile; i++)
			process.run += process.length[i];
		process.start = process.run + process.startloc;
		
		process.end = 0;
		for (i=0; i<process.endfile; i++)
			process.end += process.length[i];
		process.end += process.endloc;
		
		process.trackleft = (int)(process.start*TRACK_PITCH/process.total);
		process.trackright = (int)(process.end*TRACK_PITCH/process.total);
		
		if (preview)
		{
			Display_Flag = true;
			
			EnableMenuItem(hMenu, IDM_OPEN, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_SAVE, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_SAVE_D2V, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_LOAD_SESS, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_SAVE_SESS, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_STOP, MF_ENABLED);
			
			DisableControl(false);
			ShowStatistics(true);
		}
		
		if (saveAvi)
		{
			AVI_Flag = true;
			Display_Flag = false;
			
			EnableMenuItem(hMenu, IDM_OPEN, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_SAVE, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_SAVE_D2V, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_LOAD_SESS, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_SAVE_SESS, MF_GRAYED);
			EnableMenuItem(hMenu, IDM_STOP, MF_ENABLED);
			
			DisableControl(true);
			ShowStatistics(true);
		}

		KillFilters();
		process.locate = ((preview || saveAvi) ? LOCATE_RIP : LOCATE_INIT);
		
		if ((hThread == 0) || (WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0))
			hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);

		return true;
	}
	
	if (!AutoQuit_Flag)
		MessageBox(0, "File not found or bad file format !", "Error", MB_OK|MB_ICONSTOP);

	return false;
}

static void SaveAvi()
{
	AVI_Flag = true;
	Display_Flag = false;
	
	EnableMenuItem(hMenu, IDM_OPEN, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_SAVE, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_SAVE_D2V, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_LOAD_SESS, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_SAVE_SESS, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_STOP, MF_ENABLED);
	
	DisableControl(true);
	ShowStatistics(true);
	
	KillFilters();
	process.locate = LOCATE_RIP;
	
	if (hThread && (WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0))
		hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
}

// Processes messages for the main window
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	DWORD wmId, wmEvent;
	static HINSTANCE hLibrary;
	int i, j;

	switch (message)
	{
		case WM_CREATE:
			DragAcceptFiles(hWnd, true);

			PreScale_Ratio = 1.0; LumGain = 128;

			hDC = GetDC(hWnd);
			hBrush = CreateSolidBrush(MASKCOLOR);
			hMenu = GetMenu(hWnd);
			hProcess = GetCurrentProcess();

			for (i=0; i<MAX_FILE_NUMBER; i++)
				Infilename[i] = (char*)aligned_malloc(_MAX_PATH, 4);

			for (i=0; i<8; i++)
			{
				block[i] = (short *)aligned_malloc(sizeof(short)*64, 128);
			}

			fTempArray = (void *)aligned_malloc(sizeof(float)*128, 128);

			Initialize_REF_IDCT();
			Initialize_FPU_IDCT();

			// register VFAPI
			HKEY key; DWORD trash;

			if (RegCreateKeyEx(HKEY_CURRENT_USER, "Software\\VFPlugin", 0, "",
				REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &key, &trash) == ERROR_SUCCESS)
			{
				if (_getcwd(szBuffer, _MAX_PATH)!=NULL)
				{
					if (szBuffer[strlen(szBuffer)-1] != '\\')
						strcat(szBuffer, "\\");

					strcpy(szPath, szBuffer);

					if (_findfirst("DVD2AVI.vfp", &vfpfile) != -1L)
					{
						strcat(szBuffer, "DVD2AVI.vfp");

						RegSetValueEx(key, "DVD2AVI", 0, REG_SZ, (LPBYTE)szBuffer, strlen(szBuffer));
						CheckMenuItem(hMenu, IDM_VFAPI, MF_CHECKED);
					}

					RegCloseKey(key);
				}
			}

			// load DLL
			PlugIn_Flag = true;

			if ((hLibrary = LoadLibrary("OpenDVD.dll")) == NULL)
				PlugIn_Flag = false;
			else
			{
				if ((KeyOp = (pfnKeyOp) GetProcAddress(hLibrary, "KeyOp")) == NULL)
					PlugIn_Flag = false;

				if ((BufferOp = (pfnBufferOp) GetProcAddress(hLibrary, "BufferOp")) == NULL)
					PlugIn_Flag = false;
			}

			if (!PlugIn_Flag)
			{
				DeleteMenu(GetSubMenu(hMenu,4), 3, MF_BYPOSITION);
				DrawMenuBar(hWnd);
			}

			SetTimer(hWnd, 1, 1000, 0);  
			break;

		case WM_COMMAND:
			wmId    = LOWORD(wParam);
			wmEvent = HIWORD(wParam);

			// parse the menu selections
			switch (wmId)
			{
				case IDM_OPEN:
					DialogBox(hInst, (LPCTSTR)IDD_FILELIST, hWnd, (DLGPROC)VideoList);
					break;

				case IDM_PREVIEW:
					if (IsWindowEnabled(hTrack))
					{
						Display_Flag = true;

						EnableMenuItem(hMenu, IDM_OPEN, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_SAVE, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_SAVE_D2V, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_LOAD_SESS, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_SAVE_SESS, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_STOP, MF_ENABLED);

						DisableControl(false);
						ShowStatistics(true);

						KillFilters();
						process.locate = LOCATE_RIP;

						if (hThread && (WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0))
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
					break;

				case IDM_SAVE:
					if (PopFileDlg(szOutput, hWnd, SAVE_AVI))
					{
						SaveAvi();
					}
					break;

				case IDM_PROCESS_WAV:
					DialogBox(hInst, (LPCTSTR)IDD_FILELIST, hWnd, (DLGPROC)AudioList);
					break;

				case IDM_LOAD_D2V:
					if (PopFileDlg(szInput, hWnd, OPEN_D2V))
					{
D2V_PROCESS:
						D2VFile = fopen(szInput, "r");

						while (File_Limit)
						{
							File_Limit--;
							_close(Infile[File_Limit]);
						}

						fscanf(D2VFile, "DVD2AVIProjectFile\n%d\n", &File_Limit);

						i = File_Limit;
						while (i)
						{
							fscanf(D2VFile, "%d ", &j);
							fgets(Infilename[File_Limit-i], j+1, D2VFile);
							if ((Infile[File_Limit-i] = _open(Infilename[File_Limit-i], _O_RDONLY | _O_BINARY | _O_SEQUENTIAL))==-1)
							{
								while (i<File_Limit)
								{
									_close(Infile[File_Limit-i-1]);
									i++;
								}

								File_Limit = 0;
								break;
							}

							i--;
						}

						Recovery();

						fscanf(D2VFile, "\nStream_Type=%d,%X,%X\n", &SystemStream_Flag, &lfsr0, &lfsr1);
						if (lfsr0 || lfsr1)
							KeyOp_Flag = KEY_INPUT;
						else
							KeyOp_Flag = KEY_OFF;
						if ( SystemStream_Flag == 2 )  // if MPEG-2 transport stream is indicated...
							fscanf(D2VFile, "MPEG2_Transport_PID=%X,%X\n", 
								&MPEG2_Transport_VideoPID, &MPEG2_Transport_AudioPID );
						fscanf(D2VFile, "iDCT_Algorithm=%d\n", &iDCT_Flag);
						fscanf(D2VFile, "YUVRGB_Scale=%d\n", &Scale_Flag);
						fscanf(D2VFile, "Luminance=%d,%d\n", &LumGain, &LumOffset);


						if (hLumDlg!=NULL)
						{
							DestroyWindow(hLumDlg);
							hLumDlg = CreateDialog(hInst, (LPCTSTR)IDD_LUMINANCE, hWnd, (DLGPROC)Luminance);
						}

						fscanf(D2VFile, "Picture_Size=%d,%d,%d,%d,%d,%d\n", &Clip_Top, &Clip_Bottom, 
							&Clip_Left, &Clip_Right, &Squeeze_Width, &Squeeze_Height);

						if (hClipResizeDlg!=NULL)
						{
							DestroyWindow(hClipResizeDlg);
							hClipResizeDlg = CreateDialog(hInst, (LPCTSTR)IDD_CLIPRESIZE, hWnd, (DLGPROC)ClipResize);
						}

						fscanf(D2VFile, "Field_Operation=%d\n", &FO_Flag);
						fscanf(D2VFile, "Frame_Rate=%d\n", &i);

						CheckUncheckMenus();

						if (File_Limit)
						{
							fscanf(D2VFile, "Location=%d,%X,%d,%X\n", &process.leftfile, &process.leftlba, &process.rightfile, &process.rightlba);

							process.startfile = process.leftfile;
							process.startloc = process.leftlba * BUFFER_SIZE;
							process.endfile = process.rightfile;
							process.endloc = (process.rightlba - 1) * BUFFER_SIZE;

							process.run = 0;
							for (i=0; i<process.startfile; i++)
								process.run += process.length[i];
							process.start = process.run + process.startloc;

							process.end = 0;
							for (i=0; i<process.endfile; i++)
								process.end += process.length[i];
							process.end += process.endloc;

							process.trackleft = (int)(process.start*TRACK_PITCH/process.total);
							process.trackright = (int)(process.end*TRACK_PITCH/process.total);

							KillFilters();
							process.locate = LOCATE_INIT;

							if ((hThread == 0) || (WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0))
								hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
						}
					}
					break;

				case IDM_SAVE_D2V:
					if (PopFileDlg(szOutput, hWnd, SAVE_D2V))
					{
						sprintf(szBuffer, "%s.d2v", szOutput);

						D2VFile = fopen(szBuffer, "w+");

						D2V_Flag = true;
						Display_Flag = false;

						EnableMenuItem(hMenu, IDM_OPEN, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_SAVE, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_SAVE_D2V, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_LOAD_SESS, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_SAVE_SESS, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_STOP, MF_ENABLED);

						DisableControl(true);
						ShowStatistics(true);

						KillFilters();
						process.locate = LOCATE_RIP;

						if (hThread && (WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0))
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
					break;

				case IDM_LOAD_SESS:
					if (PopFileDlg(szInput, hWnd, OPEN_SESS))
						LoadSession(szInput, false, false);
					break;

				case IDM_SAVE_SESS:
					if (PopFileDlg(szOutput, hWnd, SAVE_SESS))
					{
						sprintf(szBuffer, "%s.d2s", szOutput);

						FILE *fd = fopen(szBuffer, "w");
						
						fprintf(fd, "DVD2AVISessionFile\n");
						i = File_Limit;
						while (i)
						{
							fprintf(fd, "AddFile=%s\n", Infilename[File_Limit-i]);
							i--;
						}
						
						fprintf(fd, "Location=%d,%X,%d,%X\n", process.leftfile, (int)process.leftlba, process.rightfile, (int)process.rightlba);
						fprintf(fd, "iDCT_Algorithm=%d\n", iDCT_Flag);
						fprintf(fd, "Field_Operation=%d\n", FO_Flag);
						fprintf(fd, "Store_Flag=%d\n", Store_Flag);
						fprintf(fd, "YUVRGB_Scale=%d\n", Scale_Flag);
						fprintf(fd, "Luminance=%d,%d\n", LumGain, LumOffset);
						fprintf(fd, "Picture_Size=%d,%d,%d,%d,%d,%d\n", Clip_Top, Clip_Bottom, Clip_Left, Clip_Right, Squeeze_Width, Squeeze_Height);
						fprintf(fd, "Resize_Function=%d\n", Resize_Function);
						fprintf(fd, "Track_Number=%d\n", Track_Flag);
						fprintf(fd, "Channel_Format=%d\n", Format_Flag);
						fprintf(fd, "AC3=%d\n", AC3_Flag);
						fprintf(fd, "DR_Control=%d\n", DRC_Flag);
						fprintf(fd, "DS_Downmix=%d\n", DSDown_Flag);
						fprintf(fd, "MPA=%d\n", MPA_Flag);
						fprintf(fd, "SRC_Precision=%d\n", SRC_Flag);
						fprintf(fd, "Norm_Ratio=%d\n", 100 * Normalization_Flag + Norm_Ratio);
						fprintf(fd, "MPEG2_Transport_PID=%X,%X\n", MPEG2_Transport_VideoPID,
							MPEG2_Transport_AudioPID );
						fprintf(fd, "Process_Priority=%d\n", Priority_Flag);

						fclose(fd);
					}
					break;

				case IDM_STOP:
					Stop_Flag = true;
					// Check if thread is still here
					if (hThread && (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0))
						hThread = 0;
					if (Pause_Flag && hThread)
						ResumeThread(hThread);
					break;

				case IDM_STORE_RGB24:
					if (Store_Flag != STORE_RGB24)
					{
						if (DDOverlay_Avail)
							IDirectDrawSurface_UpdateOverlay(lpOverlay, NULL, lpPrimary, NULL, DDOVER_HIDE, NULL);

						Store_Flag = STORE_RGB24;
						CheckMenuItem(hMenu, IDM_STORE_RGB24, MF_CHECKED);
						CheckMenuItem(hMenu, IDM_STORE_YUY2, MF_UNCHECKED);
						EnableMenuItem(hMenu, IDM_BMP, MF_ENABLED);

						KillFilters();
						RefreshWindow();
					}
					break;

				case IDM_STORE_YUY2:
					if (Store_Flag != STORE_YUY2)
					{
						Store_Flag = STORE_YUY2;
						CheckMenuItem(hMenu, IDM_STORE_RGB24, MF_UNCHECKED);
						CheckMenuItem(hMenu, IDM_STORE_YUY2, MF_CHECKED);
						EnableMenuItem(hMenu, IDM_BMP, MF_GRAYED);

						if (DDOverlay_Avail)
						{
							GetClientRect(hWnd, &crect);
							crect.bottom -= TRACK_HEIGHT;
							FillRect(hDC, &crect, hBrush);
							crect.bottom += TRACK_HEIGHT;
						}

						KillFilters();
						RefreshWindow();
					}
					break;

				case IDM_TRACK_NONE:
					ClearTrack();
					Track_Flag = TRACK_NONE;
					CheckMenuItem(hMenu, IDM_TRACK_NONE, MF_CHECKED);
					break;

				case IDM_TRACK_1:
					ClearTrack();
					Track_Flag = TRACK_1;
					CheckMenuItem(hMenu, IDM_TRACK_1, MF_CHECKED);
					break;

				case IDM_TRACK_2:
					ClearTrack();
					Track_Flag = TRACK_2;
					CheckMenuItem(hMenu, IDM_TRACK_2, MF_CHECKED);
					break;

				case IDM_TRACK_3:
					ClearTrack();
					Track_Flag = TRACK_3;
					CheckMenuItem(hMenu, IDM_TRACK_3, MF_CHECKED);
					break;

				case IDM_TRACK_4:
					ClearTrack();
					Track_Flag = TRACK_4;
					CheckMenuItem(hMenu, IDM_TRACK_4, MF_CHECKED);
					break;

				case IDM_TRACK_5:
					ClearTrack();
					Track_Flag = TRACK_5;
					CheckMenuItem(hMenu, IDM_TRACK_5, MF_CHECKED);
					break;

				case IDM_TRACK_6:
					ClearTrack();
					Track_Flag = TRACK_6;
					CheckMenuItem(hMenu, IDM_TRACK_6, MF_CHECKED);
					break;

				case IDM_TRACK_7:
					ClearTrack();
					Track_Flag = TRACK_7;
					CheckMenuItem(hMenu, IDM_TRACK_7, MF_CHECKED);
					break;

				case IDM_TRACK_8:
					ClearTrack();
					Track_Flag = TRACK_8;
					CheckMenuItem(hMenu, IDM_TRACK_8, MF_CHECKED);
					break;

				case IDM_AC3:
					Format_Flag = FORMAT_AC3;
					CheckMenuItem(hMenu, IDM_AC3, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_MPA, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LPCM, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SELECT, MF_UNCHECKED);
					break;

				case IDM_MPA:
					Format_Flag = FORMAT_MPA;
					CheckMenuItem(hMenu, IDM_AC3, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_MPA, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_LPCM, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SELECT, MF_UNCHECKED);
					break;

				case IDM_LPCM:
					Format_Flag = FORMAT_LPCM;
					CheckMenuItem(hMenu, IDM_AC3, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_MPA, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LPCM, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_SELECT, MF_UNCHECKED);
					break;

				case IDM_AC3_DEMUXALL:
					AC3_Flag = AUDIO_DEMUXALL;
					CheckMenuItem(hMenu, IDM_AC3_DEMUXALL, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_AC3_DEMUXONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_AC3_DECODE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_AC3_BESWEET,MF_UNCHECKED);
					break;

				case IDM_AC3_DEMUXONE:
					AC3_Flag = AUDIO_DEMUXONE;
					CheckMenuItem(hMenu, IDM_AC3_DEMUXALL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_AC3_DEMUXONE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_AC3_DECODE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_AC3_BESWEET,MF_UNCHECKED);
					break;

				case IDM_AC3_DECODE:
					AC3_Flag = AUDIO_DECODE;
					CheckMenuItem(hMenu, IDM_AC3_DEMUXALL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_AC3_DEMUXONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_AC3_DECODE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_AC3_BESWEET,MF_UNCHECKED);
					break;
				//Preliminary Variable and GUI stuff for BeSweet interface
				//01/26/02 int21h
				case IDM_AC3_BESWEET:
					AC3_Flag = AUDIO_BESWEET;
					CheckMenuItem(hMenu, IDM_AC3_DEMUXALL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_AC3_DEMUXONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_AC3_DECODE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_AC3_BESWEET,MF_CHECKED);
					break;

				case IDM_DRC_NONE:
					DRC_Flag = DRC_NONE;
					CheckMenuItem(hMenu, IDM_DRC_NONE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_DRC_LIGHT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_NORMAL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_HEAVY, MF_UNCHECKED);
					break;

				case IDM_DRC_LIGHT:
					DRC_Flag = DRC_LIGHT;
					CheckMenuItem(hMenu, IDM_DRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_LIGHT, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_DRC_NORMAL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_HEAVY, MF_UNCHECKED);
					break;

				case IDM_DRC_NORMAL:
					DRC_Flag = DRC_NORMAL;
					CheckMenuItem(hMenu, IDM_DRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_LIGHT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_NORMAL, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_DRC_HEAVY, MF_UNCHECKED);
					break;

				case IDM_DRC_HEAVY:
					DRC_Flag = DRC_HEAVY;
					CheckMenuItem(hMenu, IDM_DRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_LIGHT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_NORMAL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_HEAVY, MF_CHECKED);
					break;

				case IDM_DSDOWN:
					if (DSDown_Flag)
						CheckMenuItem(hMenu, IDM_DSDOWN, MF_UNCHECKED);
					else
						CheckMenuItem(hMenu, IDM_DSDOWN, MF_CHECKED);

					DSDown_Flag = !DSDown_Flag;
					break;

				case IDM_PRESCALE:
					if (PreScale_Ratio!=1.0 || !Check_Flag || !IsWindowEnabled(hTrack))
					{
						CheckMenuItem(hMenu, IDM_PRESCALE, MF_UNCHECKED);
						PreScale_Ratio = 1.0;
					}
					else
					{
						Decision_Flag = true;
						Display_Flag = false;

						EnableMenuItem(hMenu, IDM_OPEN, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_SAVE, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_SAVE_D2V, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_LOAD_SESS, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_SAVE_SESS, MF_GRAYED);
						EnableMenuItem(hMenu, IDM_STOP, MF_ENABLED);

						DisableControl(true);
						ShowStatistics(true);

						KillFilters();
						process.locate = LOCATE_RIP;
						PreScale_Ratio = 1.0;

						if (hThread && (WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0))
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
					break;

				case IDM_MPA_DEMUXALL:
					MPA_Flag = AUDIO_DEMUXALL;
					CheckMenuItem(hMenu, IDM_MPA_DEMUXALL, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_MPA_DEMUXONE, MF_UNCHECKED);
					break;

				case IDM_MPA_DEMUXONE:
					MPA_Flag = AUDIO_DEMUXONE;
					CheckMenuItem(hMenu, IDM_MPA_DEMUXALL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_MPA_DEMUXONE, MF_CHECKED);
					break;

				case IDM_SELECT:
					Format_Flag = FORMAT_AUTO;
					CheckMenuItem(hMenu, IDM_AC3, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_MPA, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LPCM, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SELECT, MF_CHECKED);
					break;

				case IDM_IDCT_MMX:
					iDCT_Flag = IDCT_MMX;
					CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_REF, MF_UNCHECKED);
					break;

				case IDM_IDCT_SSEMMX:
					iDCT_Flag = IDCT_SSEMMX;
					CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_REF, MF_UNCHECKED);
					break;

				case IDM_IDCT_SSE2MMX:
					iDCT_Flag = IDCT_SSE2MMX;
					CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_REF, MF_UNCHECKED);
					break;


				case IDM_IDCT_FPU:
					iDCT_Flag = IDCT_FPU;
					CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_REF, MF_UNCHECKED);
					break;

				case IDM_IDCT_REF:
					iDCT_Flag = IDCT_REF;
					CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_REF, MF_CHECKED);
					break;

				case IDM_FO_NONE:
					FO_Flag = FO_NONE;
					CheckMenuItem(hMenu, IDM_FO_NONE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_FO_FILM, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_FO_SWAP, MF_UNCHECKED);
					SetDlgItemText(hDlg, IDC_INFO, "");
					break;

				case IDM_FO_FILM:
					FO_Flag = FO_FILM;
					CheckMenuItem(hMenu, IDM_FO_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_FO_FILM, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_FO_SWAP, MF_UNCHECKED);
					SetDlgItemText(hDlg, IDC_INFO, "");
					break;

				case IDM_FO_SWAP:
					FO_Flag = FO_SWAP;
					CheckMenuItem(hMenu, IDM_FO_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_FO_FILM, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_FO_SWAP, MF_CHECKED);
					SetDlgItemText(hDlg, IDC_INFO, "");
					break;

				case IDM_TVSCALE:
					RGB_Scale = 0x1000200010002000;
					RGB_Offset = 0x0000000000000000;
					RGB_CBU = 0x000038B4000038B4;
					RGB_CGX = 0xF4FDE926F4FDE926;
					RGB_CRV = 0x00002CDD00002CDD;

					Scale_Flag = false;
					CheckMenuItem(hMenu, IDM_TVSCALE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_PCSCALE, MF_UNCHECKED);

					RefreshWindow();
					break;

				case IDM_PCSCALE:
					RGB_Scale = 0x1000254310002543;
					RGB_Offset = 0x0010001000100010;
					RGB_CBU = 0x0000408D0000408D;
					RGB_CGX = 0xF377E5FCF377E5FC;
					RGB_CRV = 0x0000331300003313;

					Scale_Flag = true;
					CheckMenuItem(hMenu, IDM_TVSCALE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_PCSCALE, MF_CHECKED);

					RefreshWindow();
					break;

				case IDM_CLIPRESIZE_:
					if (hClipResizeDlg!=NULL)
					{
						DestroyWindow(hClipResizeDlg);
						hClipResizeDlg = NULL;
					}
					else
						hClipResizeDlg = CreateDialog(hInst, (LPCTSTR)IDD_CLIPRESIZE, hWnd, (DLGPROC)ClipResize);
					break;

				case IDM_LUMINANCE_:
					if (hLumDlg!=NULL)
					{
						DestroyWindow(hLumDlg);
						hLumDlg = NULL;
					}
					else
						hLumDlg = CreateDialog(hInst, (LPCTSTR)IDD_LUMINANCE, hWnd, (DLGPROC)Luminance);
					break;

				case IDM_NORM:
					if (hNormDlg!=NULL)
					{
						DestroyWindow(hNormDlg);
						hNormDlg = NULL;
					}
					else
						hNormDlg = CreateDialog(hInst, (LPCTSTR)IDD_NORM, hWnd, (DLGPROC)Normalization);
					break;

				case IDM_PP_HIGH:
					Priority_Flag = PRIORITY_HIGH;
					SetPriorityClass(hProcess, HIGH_PRIORITY_CLASS);
					CheckMenuItem(hMenu, IDM_PP_HIGH, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_PP_NORMAL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_PP_LOW, MF_UNCHECKED);
					break;

				case IDM_PP_NORMAL:
					Priority_Flag = PRIORITY_NORMAL;
					SetPriorityClass(hProcess, NORMAL_PRIORITY_CLASS);
					CheckMenuItem(hMenu, IDM_PP_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_PP_NORMAL, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_PP_LOW, MF_UNCHECKED);
					break;

				case IDM_PP_LOW:
					Priority_Flag = PRIORITY_LOW;
					SetPriorityClass(hProcess, IDLE_PRIORITY_CLASS);
					CheckMenuItem(hMenu, IDM_PP_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_PP_NORMAL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_PP_LOW, MF_CHECKED);
					break;

				case IDM_PAUSE:
					// Check if thread is still here
					if (hThread && (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0))
						hThread = 0;
					if (hThread)
					{
						if (Pause_Flag)
							ResumeThread(hThread);
						else
							SuspendThread(hThread);
						
						Pause_Flag = !Pause_Flag;
					}
					break;

				case IDM_STATISTICS:
					if (Statistics_Flag)
					{
						DestroyWindow(hDlg);
						Statistics_Flag = false;
						hDlg = NULL;
					}
					else
						ShowStatistics(true);
					break;

				case IDM_DISPLAY:
					Display_Flag = !Display_Flag;
					break;

				case IDM_BMP:
					int width, height;

					width = Coded_Picture_Width;
					height = Coded_Picture_Height;

						width -= Clip_Left+Clip_Right;
						height -= Clip_Top+Clip_Bottom;

					SYSTEMTIME st;
					GetLocalTime(&st);

					sprintf(szTemp, "%sDVD2AVI %02d%02d%02d%02d%02d.bmp", szPath, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
					BMPFile = fopen(szTemp, "wb");

					i = fwrite(BMPHeader, 1, sizeof(BMPHeader), BMPFile);
					i += fwrite(rgb24, 1, width * height * 3, BMPFile);

					j = i & 3;

					while (j>0)
					{
						i += fputc(0, BMPFile);
						j--;
					}

					fseek(BMPFile, 2, SEEK_SET);
					fwrite(&i, sizeof(int), 1, BMPFile);
					fseek(BMPFile, 18, SEEK_SET);
					fwrite(&width, sizeof(int), 1, BMPFile);
					fwrite(&height, sizeof(int), 1, BMPFile);
					
					fclose(BMPFile);
					break;

				case IDM_SRC_NONE:
					SRC_Flag = SRC_NONE;
					CheckMenuItem(hMenu, IDM_SRC_NONE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_SRC_LOW, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_MID, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_UNCHECKED);
					break;

				case IDM_SRC_LOW:
					SRC_Flag = SRC_LOW;
					CheckMenuItem(hMenu, IDM_SRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_LOW, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_SRC_MID, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_UNCHECKED);
					break;

				case IDM_SRC_MID:
					SRC_Flag = SRC_MID;
					CheckMenuItem(hMenu, IDM_SRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_LOW, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_MID, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_UNCHECKED);
					break;

				case IDM_SRC_HIGH:
					SRC_Flag = SRC_HIGH;
					CheckMenuItem(hMenu, IDM_SRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_LOW, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_MID, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_UNCHECKED);
					break;

				case IDM_SRC_UHIGH:
					SRC_Flag = SRC_UHIGH;
					CheckMenuItem(hMenu, IDM_SRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_LOW, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_MID, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_CHECKED);
					break;

				case IDM_KEY_OFF:
					KeyOp_Flag = KEY_OFF;
					CheckMenuItem(hMenu, IDM_KEY_OFF, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_KEY_INPUT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_KEY_OP, MF_UNCHECKED);
					break;

				case IDM_KEY_INPUT:
					KeyOp_Flag = KEY_INPUT;
					CheckMenuItem(hMenu, IDM_KEY_OFF, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_KEY_INPUT, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_KEY_OP, MF_UNCHECKED);
					break;

				case IDM_KEY_OP:
					KeyOp_Flag = KEY_OP;
					CheckMenuItem(hMenu, IDM_KEY_OFF, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_KEY_INPUT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_KEY_OP, MF_CHECKED);
					break;

				case IDM_ABOUT:
					DialogBox(hInst, (LPCTSTR)IDD_ABOUT, hWnd, (DLGPROC)About);
					break;

				case IDM_HOMEPAGE:
					ShellExecute(NULL, "open", "http://hiroko.ee.ntu.edu.tw/", NULL, NULL, SW_SHOWNORMAL);
					break;

				case IDM_EXIT:
					Stop_Flag = true;
					AutoQuit_Flag = true;
					// Check if thread is still here
					if (hThread && (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0))
						hThread = 0;
					if (hThread)
					{
						if (Pause_Flag)
							ResumeThread(hThread);
					}
					else
						DestroyWindow(hWnd);
					break;

				case ID_LEFT_BUTTON:
					if (IsWindowEnabled(hTrack))
					{
						SetFocus(hWnd);

						if ((process.file < process.rightfile) || (process.file==process.rightfile && process.lba<process.rightlba))
						{
							process.leftfile = process.file;
							process.leftlba = process.lba;

							process.run = 0;
							for (i=0; i<process.leftfile; i++)
								process.run += process.length[i];
							process.trackleft = (int)((process.run + (__int64)process.leftlba * BUFFER_SIZE) * TRACK_PITCH / process.total);

							SendMessage(hTrack, TBM_SETPOS, (WPARAM) true, process.trackleft);
							SendMessage(hTrack, TBM_SETSEL, (WPARAM) true, (LPARAM) MAKELONG(process.trackleft, process.trackright));
						}
					}
					break;

				case ID_LEFT_ARROW:
					SetFocus(hWnd);

					if (hThread && (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0))
					{
						Display_Flag = true;

						process.locate = LOCATE_BACKWARD;
						hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
					break;

				case ID_RIGHT_ARROW:
					SetFocus(hWnd);

					if (hThread && WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
					{
						Display_Flag = true;

						process.locate = LOCATE_FORWARD;
						hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
					break;

				case ID_RIGHT_BUTTON:
					if (IsWindowEnabled(hTrack))
					{
						SetFocus(hWnd);

						if ((process.file>process.leftfile) || (process.file==process.leftfile && process.lba>process.leftlba))
						{
							process.rightfile = process.file;
							process.rightlba = process.lba;

							process.run = 0;
							for (i=0; i<process.rightfile; i++)
								process.run += process.length[i];
							process.trackright = (int)((process.run + (__int64)process.rightlba*BUFFER_SIZE)*TRACK_PITCH/process.total);

							SendMessage(hTrack, TBM_SETPOS, (WPARAM) true, process.trackright);
							SendMessage(hTrack, TBM_SETSEL, (WPARAM) true, (LPARAM) MAKELONG(process.trackleft, process.trackright));
						}
					}
					break;

				default:
					return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;

		case WM_HSCROLL:
			SetFocus(hWnd);

			if (hThread && WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
			{
				Display_Flag = true;

				process.startloc = process.start = process.total*SendMessage(hTrack, TBM_GETPOS, 0, 0)/TRACK_PITCH;

				process.startfile = 0; process.run = 0;
				while (process.startloc > process.length[process.startfile])
				{
					process.startloc -= process.length[process.startfile];
					process.run += process.length[process.startfile];
					process.startfile++;
				}

				process.end = process.total - BUFFER_SIZE;
				process.endfile = File_Limit - 1;
				process.endloc = (process.length[File_Limit-1]/BUFFER_SIZE-1)*BUFFER_SIZE;

				process.locate = LOCATE_SCROLL;

				hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
			}
			break;

		case WM_KEYDOWN:
			if (IsWindowEnabled(hTrack))
			{
				switch (wParam)
				{
					case VK_LEFT:
						if (hThread && WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
						{
							Display_Flag = true;

							process.locate = LOCATE_BACKWARD;
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
						}
						break;

					case VK_RIGHT:
						if (hThread && WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
						{
							Display_Flag = true;

							process.locate = LOCATE_FORWARD;
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
						}
						break;
				}
			}
			break;

		case WM_SIZE:
			if (!IsIconic(hWnd))
				ShowStatistics(false);
			else if (DDOverlay_Avail && Check_Flag)
				IDirectDrawSurface_UpdateOverlay(lpOverlay, NULL, lpPrimary, NULL, DDOVER_HIDE, NULL);
			break;

		case WM_MOVE:
			if (!IsIconic(hWnd))
			{
				ShowStatistics(false);

				if (Check_Flag)
				{
					if (DDOverlay_Avail && (Store_Flag == STORE_YUY2))
						ResizeOverlay();
					else if (Store_Flag == STORE_RGB24)
						RenderRGB24(GetFilteredOutput());
				}
			}
			break;

		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			EndPaint(hWnd, &ps);
			ReleaseDC(hWnd, hdc);

			if (Check_Flag && (Store_Flag == STORE_RGB24))
				RenderRGB24(GetFilteredOutput());
			break;

		case WM_DROPFILES:
			char seq, *ext;

			DragQueryFile((HDROP)wParam, 0, szInput, sizeof(szInput));
			DragFinish((HDROP)wParam);
			SetForegroundWindow(hWnd);

			ext = strrchr(szInput, '.');
			if (ext!=NULL)
			{
				if (!_strnicmp(ext, ".d2v", 4))
					goto D2V_PROCESS;

				if (!_strnicmp(ext, ".d2s", 4))
					LoadSession(szInput, false, false);

				if (!_strnicmp(ext, ".wav", 4))
				{
					if (Check_Flag)
						break;

					if (!CheckWAV())
					{
						DialogBox(hInst, (LPCTSTR)IDD_ERROR, hWnd, (DLGPROC)About);
						break;
					}

					if (PopFileDlg(szOutput, hWnd, SAVE_WAV))
					{
						DialogBox(hInst, (LPCTSTR)IDD_DELAY, hWnd, (DLGPROC)Delay);
						ShowStatistics(true);

						if ((hThread == 0) || WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
							hThread = CreateThread(NULL, 0, ProcessWAV, 0, 0, &threadId);
					}
					break;
				}
			}

			while (File_Limit)
			{
				File_Limit--;
				_close(Infile[File_Limit]);
			}

			while (_findfirst(szInput, &seqfile) != -1L)
			{
				strcpy(Infilename[File_Limit], szInput);
				Infile[File_Limit] = _open(szInput, _O_RDONLY | _O_BINARY | _O_SEQUENTIAL);
				File_Limit++;

				strncpy(&seq, szInput+strlen(szInput)-5, 1);
				if (seq>='0' && seq<'9')
					seq++;
				else
					break;
				strncpy(szInput+strlen(szInput)-5, &seq, 1);
			}

			Recovery();

			if (File_Limit)
			{
				process.rightfile = File_Limit-1;
				process.rightlba = (int)(process.length[File_Limit-1]/BUFFER_SIZE);

				process.end = process.total - BUFFER_SIZE;
				process.endfile = File_Limit - 1;
				process.endloc = (process.length[File_Limit-1]/BUFFER_SIZE - 1)*BUFFER_SIZE;

				KillFilters();
				process.locate = LOCATE_INIT;

				if ((hThread == 0) || WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
					hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
			}
			break;

		case WM_TIMER:
			if (Quit_Flag)
			{
				if ((hThread == 0) || WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
				{
					hThread = 0;
					DestroyWindow(hWnd);
				}
			}
			break;

		case WM_CLOSE:
			PostMessage(hWnd, WM_COMMAND, IDM_EXIT, 0);
			break;

		case WM_DESTROY:
			strcat(szPath, "DVD2AVI.ini");
			GetWindowRect(hWnd, &wrect);

			if ((INIFile = fopen(szPath, "w")) != NULL)
			{
				fprintf(INIFile, "INI_Version=%d\n", INI_Version);
				fprintf(INIFile, "Window_Position=%d,%d\n", wrect.left, wrect.top);
				fprintf(INIFile, "iDCT_Algorithm=%d\n", iDCT_Flag);
				fprintf(INIFile, "YUVRGB_Scale=%d\n", Scale_Flag);
				fprintf(INIFile, "Field_Operation=%d\n", FO_Flag);
				fprintf(INIFile, "Track_Number=%d\n", Track_Flag);
				fprintf(INIFile, "Channel_Format=%d\n", Format_Flag);
				fprintf(INIFile, "AC3=%d\n", AC3_Flag);
				fprintf(INIFile, "DR_Control=%d\n", DRC_Flag);
				fprintf(INIFile, "DS_Downmix=%d\n", DSDown_Flag);
				fprintf(INIFile, "MPA=%d\n", MPA_Flag);
				fprintf(INIFile, "SRC_Precision=%d\n", SRC_Flag);
				fprintf(INIFile, "Norm_Ratio=%d\n", 100 * Normalization_Flag + Norm_Ratio);
				fprintf(INIFile, "MPEG2_Transport_PID=%X,%X\n", 
					MPEG2_Transport_VideoPID, MPEG2_Transport_AudioPID );
				fprintf(INIFile, "Process_Priority=%d\n", Priority_Flag);
				fprintf(INIFile, "Store_Flag=%d\n", Store_Flag);

				fclose(INIFile);
			}

			while (File_Limit)
			{
				File_Limit--;
				_close(Infile[File_Limit]);
			}

			Recovery();

			for (i=0; i<8; i++)
				aligned_free(block[i]);
			aligned_free(fTempArray);

			for (i=0; i<MAX_FILE_NUMBER; i++)
				aligned_free(Infilename[i]);

			if (hLibrary)
				FreeLibrary(hLibrary);

			ReleaseDC(hWnd, hDC);
			DeleteObject(hBrush);

			PostQuitMessage(0);
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return false;
}

LRESULT CALLBACK VideoList(HWND hVideoListDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int i, j;

	switch (message)
	{
		case WM_INITDIALOG:
			if (File_Limit)
				for (i=0; i<File_Limit; i++)
					SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_ADDSTRING, 0, (LPARAM)Infilename[i]);
			else
				OpenVideoFile(hVideoListDlg);

			if (File_Limit)
				SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_SETCURSEL, File_Limit-1, 0);

			return true;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case ID_ADD:
					OpenVideoFile(hVideoListDlg);

					if (File_Limit)
						SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_SETCURSEL, File_Limit-1, 0);
					break;

				case ID_DEL:
					if (File_Limit)
					{
						i= SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_GETCURSEL, 0, 0);
						SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_DELETESTRING, i, 0);

						File_Limit--;
						_close(Infile[i]);

						for (j=i; j<File_Limit; j++)
						{
							Infile[j] = Infile[j+1];
							strcpy(Infilename[j], Infilename[j+1]);
						}

						SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_SETCURSEL, i>=File_Limit ? File_Limit-1 : i, 0);
					}
					break;

				case IDOK:
				case IDCANCEL:
					EndDialog(hVideoListDlg, 0);
					Recovery();

					if (File_Limit)
					{
						process.rightfile = File_Limit-1;
						process.rightlba = (int)(process.length[File_Limit-1]/BUFFER_SIZE);

						process.end = process.total - BUFFER_SIZE;
						process.endfile = File_Limit - 1;
						process.endloc = (process.length[File_Limit-1]/BUFFER_SIZE - 1)*BUFFER_SIZE;

						KillFilters();
						process.locate = LOCATE_INIT;

						if ((hThread == 0) || WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
					return true;
			}
			break;
	}
    return false;
}

static void OpenVideoFile(HWND hVideoListDlg)
{
	char seq;

	if (PopFileDlg(szInput, hVideoListDlg, OPEN_VOB))
	{
		while (_findfirst(szInput, &seqfile) != -1L)
		{
			SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_ADDSTRING, 0, (LPARAM)szInput);
			strcpy(Infilename[File_Limit], szInput);
			Infile[File_Limit] = _open(szInput, _O_RDONLY | _O_BINARY | _O_SEQUENTIAL);
			File_Limit++;

			strncpy(&seq, szInput+strlen(szInput)-5, 1);
			if (seq>='0' && seq<'9')
				seq++;
			else
				break;
			strncpy(szInput+strlen(szInput)-5, &seq, 1);
		}
	}
}

LRESULT CALLBACK AudioList(HWND hAudioListDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int i, j;

	switch (message)
	{
		case WM_INITDIALOG:
			File_Limit = 0;
			OpenAudioFile(hAudioListDlg);
			return true;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case ID_ADD:
					OpenAudioFile(hAudioListDlg);
					break;

				case ID_DEL:
					if (File_Limit)
					{
						i= SendDlgItemMessage(hAudioListDlg, IDC_LIST, LB_GETCURSEL, 0, 0);
						SendDlgItemMessage(hAudioListDlg, IDC_LIST, LB_DELETESTRING, i, 0);

						File_Limit--;

						for (j=i; j<File_Limit; j++)
						{
							strcpy(Infilename[j], Infilename[j+1]);
							strcpy(Outfilename[j], Outfilename[j+1]);
							SoundDelay[j] = SoundDelay[j+1];
						}

						SendDlgItemMessage(hAudioListDlg, IDC_LIST, LB_SETCURSEL, i>=File_Limit ? File_Limit-1 : i, 0);
					}
					break;

				case IDOK:
				case IDCANCEL:
					EndDialog(hAudioListDlg, 0);

					if (File_Limit)
					{
						ShowStatistics(true);

						if ((hThread == 0) || WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
							hThread = CreateThread(NULL, 0, ProcessWAV, (void *)File_Limit, 0, &threadId);
					}
					return true;
			}
			break;
	}
    return false;
}

static void OpenAudioFile(HWND hAudioListDlg)
{
	if (PopFileDlg(szInput, hAudioListDlg, OPEN_WAV))
	{
		if (!CheckWAV())
		{
			DialogBox(hInst, (LPCTSTR)IDD_ERROR, hAudioListDlg, (DLGPROC)About);
			return;
		}

		if (PopFileDlg(szOutput, hAudioListDlg, SAVE_WAV))
		{
			strcpy(Infilename[File_Limit], szInput);
			strcpy(Outfilename[File_Limit], szOutput);
			DialogBox(hInst, (LPCTSTR)IDD_DELAY, hWnd, (DLGPROC)Delay);
			sprintf(szBuffer, "%s %dms", szInput, SoundDelay[File_Limit]);
			SendDlgItemMessage(hAudioListDlg, IDC_LIST, LB_ADDSTRING, 0, (LPARAM)szBuffer);
			File_Limit++;
		}
	}

	if (File_Limit)
		SendDlgItemMessage(hAudioListDlg, IDC_LIST, LB_SETCURSEL, File_Limit-1, 0);
}

DWORD WINAPI ProcessWAV(LPVOID n)
{
	int i;

	Stop_Flag = Pause_Flag = false;

	EnableMenuItem(hMenu, IDM_OPEN, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_PROCESS_WAV, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_LOAD_SESS, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_STOP, MF_ENABLED);

	EnableMenuItem(hMenu, 2, MF_BYPOSITION | MF_GRAYED);
	DrawMenuBar(hWnd);

	DragAcceptFiles(hWnd, false);

	if (!n)
		Wavefs44File(SoundDelay[0], 1, 1);
	else
		for (i=0; i<File_Limit && !Stop_Flag; i++)
		{
			strcpy(szInput, Infilename[i]);
			strcpy(szOutput, Outfilename[i]);
			Wavefs44File(SoundDelay[i], i+1, File_Limit);
		}

	File_Limit = 0;

	EnableMenuItem(hMenu, IDM_OPEN, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_PROCESS_WAV, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_LOAD_SESS, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_STOP, MF_GRAYED);

	EnableMenuItem(hMenu, 2, MF_BYPOSITION | MF_ENABLED);
	DrawMenuBar(hWnd);

	if (!Stop_Flag)
	{
		MessageBeep(MB_OK);
		SetDlgItemText(hDlg, IDC_REMAIN, "FINISH");
	}
	
	SetForegroundWindow(hWnd);
	DragAcceptFiles(hWnd, true);
	return 0;
}

void ThreadKill()
{
	KillFilters();

	if ((AVI_Flag || D2V_Flag) && Track_Flag!=TRACK_NONE && 
		((ac3[Track_Flag].rip && AC3_Flag==AUDIO_DECODE) || pcm.rip))
	{
		if (SRC_Flag)
		{
			EndSRC(pcm.file);
			pcm.size = ((int)(0.91875*pcm.size)>>2)<<2;
		}

		Normalize(NULL, 44, pcm.filename, pcm.file, 44, pcm.size);
		EndWAV(pcm.file, pcm.size);
	}

	if (process.locate==LOCATE_INIT || process.locate==LOCATE_RIP)
	{
		if (D2V_Flag)
		{
			if (Stop_Flag)
				fprintf(D2VFile, " 9\n\nINTERRUPTED");
			else
				fprintf(D2VFile, " 9\n\nFINISHED");
		}

		_fcloseall();

		if (Decision_Flag)
		{
			if (Sound_Max > 1)
			{
				PreScale_Ratio = 327.68 * Norm_Ratio / Sound_Max;

				if (PreScale_Ratio > 1.0 && PreScale_Ratio < 1.01)
					PreScale_Ratio = 1.0;

				sprintf(szBuffer, "%.2f", PreScale_Ratio);
				SetDlgItemText(hDlg, IDC_INFO, szBuffer);

				CheckMenuItem(hMenu, IDM_PRESCALE, MF_CHECKED);
				CheckMenuItem(hMenu, IDM_NORM, MF_UNCHECKED);
				Normalization_Flag = false;
			}
			else
			{
				SetDlgItemText(hDlg, IDC_INFO, "N.A.");
				CheckMenuItem(hMenu, IDM_PRESCALE, MF_UNCHECKED);
			}
		}

		AVI_Flag = false;
		D2V_Flag = false;
		Decision_Flag = false;
		Display_Flag = false;

		EnableMenuItem(hMenu, IDM_OPEN, MF_ENABLED);
		EnableMenuItem(hMenu, IDM_SAVE, MF_ENABLED);
		EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_ENABLED);
		EnableMenuItem(hMenu, IDM_SAVE_D2V, MF_ENABLED);
		EnableMenuItem(hMenu, IDM_LOAD_SESS, MF_ENABLED);
		EnableMenuItem(hMenu, IDM_SAVE_SESS, MF_ENABLED);
		EnableMenuItem(hMenu, IDM_STOP, MF_GRAYED);

		EnableMenuItem(hMenu, 1, MF_BYPOSITION | MF_ENABLED);
		EnableMenuItem(hMenu, 2, MF_BYPOSITION | MF_ENABLED);
		DrawMenuBar(hWnd);

		EnableWindow(hTrack, true);
		EnableWindow(hLeftButton, true);
		EnableWindow(hLeftArrow, true);
		EnableWindow(hRightArrow, true);
		EnableWindow(hRightButton, true);
		SendMessage(hTrack, TBM_SETSEL, (WPARAM) true, (LPARAM) MAKELONG(process.trackleft, process.trackright));
		DragAcceptFiles(hWnd, true);
	}

	if (process.locate==LOCATE_RIP)
	{
		SetForegroundWindow(hWnd);

		if (!Stop_Flag)
		{
			MessageBeep(MB_OK);	
			SetDlgItemText(hDlg, IDC_REMAIN, "FINISH");
		}
	}

	if (AutoQuit_Flag)
		Quit_Flag = true;

	ExitThread(0);
}

LRESULT CALLBACK Statistics(HWND hStatisticsDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			SendDlgItemMessage(hStatisticsDlg, IDC_PROGRESS, PBM_SETRANGE, 0, MAKELPARAM(0, 10000));
			return true;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDCANCEL)
			{
				DestroyWindow(hStatisticsDlg);
				Statistics_Flag = false;
				return true;
			}
	}
    return false;
}

LRESULT CALLBACK Delay(HWND hDelayDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			SetDlgItemText(hDelayDlg, IDC_DELAY, "0");
			return true;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
			{
				SoundDelay[File_Limit] = GetDlgItemInt(hDelayDlg, IDC_DELAY, NULL, true);
				if (abs(SoundDelay[File_Limit]) > 10000000)
					SoundDelay[File_Limit] = 0;

				EndDialog(hDelayDlg, 0);
				return true;
			}
	}
    return false;
}

LRESULT CALLBACK About(HWND hAboutDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			return true;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
			{
				EndDialog(hAboutDlg, 0);
				return true;
			}
	}
    return false;
}

void UpdateSizes(HWND hDialog)
{
	SendDlgItemMessage(hDialog, IDC_LEFT_SLIDER, TBM_SETPOS, 1, Clip_Left>>3);
	sprintf(szTemp, "%d", Clip_Left);
	SetDlgItemText(hDialog, IDC_LEFT, szTemp);
	
	SendDlgItemMessage(hDialog, IDC_RIGHT_SLIDER, TBM_SETPOS, 1, Clip_Right>>3);
	sprintf(szTemp, "%d", Clip_Right);
	SetDlgItemText(hDialog, IDC_RIGHT, szTemp);
	
	SendDlgItemMessage(hDialog, IDC_TOP_SLIDER, TBM_SETPOS, 1, Clip_Top>>1);
	sprintf(szTemp, "%d", Clip_Top);
	SetDlgItemText(hDialog, IDC_TOP, szTemp);
	
	SendDlgItemMessage(hDialog, IDC_BOTTOM_SLIDER, TBM_SETPOS, 1, Clip_Bottom>>1);
	sprintf(szTemp, "%d", Clip_Bottom);
	SetDlgItemText(hDialog, IDC_BOTTOM, szTemp);
	
	SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width)>>1);
	sprintf(szTemp, "%d", Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width);
	SetDlgItemText(hDialog, IDC_WIDTH, szTemp);
	
	SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height)>>3);
	sprintf(szTemp, "%d", Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height);
	SetDlgItemText(hDialog, IDC_HEIGHT, szTemp);
	
	switch(Aspect_Change)
	{
	case ASPECT_FREE:
		EnableWindow(GetDlgItem(hDialog, IDC_HEIGHT_SLIDER), true);
		EnableWindow(GetDlgItem(hDialog, IDC_TOP_SLIDER), true);
		EnableWindow(GetDlgItem(hDialog, IDC_BOTTOM_SLIDER), true);
		break;
	case ASPECT_43:
	case ASPECT_185:
	case ASPECT_235:
		EnableWindow(GetDlgItem(hDialog, IDC_HEIGHT_SLIDER), false);
		EnableWindow(GetDlgItem(hDialog, IDC_TOP_SLIDER), false);
		EnableWindow(GetDlgItem(hDialog, IDC_BOTTOM_SLIDER), false);
		break;
	}
	SendDlgItemMessage(hDialog, IDC_ASPECT, CB_SETCURSEL, Aspect_Change, 0);
}

void AutoSize()
{
	if (Aspect_Change == ASPECT_FREE)
		return;

	double width = Coded_Picture_Width - Clip_Left - Clip_Right - Squeeze_Width;

	double input_aspect = 1.0;
	double movie_aspect = 1.0;
	double pc_aspect = (4./3.) * (width / (double)Coded_Picture_Width);

	switch(aspect_ratio_information)
	{
	case 2:
		input_aspect = (4./3.) * ((double)(Coded_Picture_Width - Clip_Left - Clip_Right) / (double)Coded_Picture_Width);
		break;
	case 3:
		input_aspect = (16./9.) * ((double)(Coded_Picture_Width - Clip_Left - Clip_Right) / (double)Coded_Picture_Width);
		break;
	case 4:
		input_aspect = (2.21) * ((double)(Coded_Picture_Width - Clip_Left - Clip_Right) / (double)Coded_Picture_Width);
		break;
	}

	switch(Aspect_Change)
	{
	case ASPECT_43:
		movie_aspect = 4./3.;
		break;
	case ASPECT_185:
		movie_aspect = 1.85;
		break;
	case ASPECT_235:
		movie_aspect = 2.35;
		break;
	}

	double crop = ((double)Coded_Picture_Height - ((double)Coded_Picture_Height / (movie_aspect/input_aspect)));
	if (crop<0)
		crop=0;
	double crop_err = crop;
	crop = ((int)crop) & ~15;
	double height = ((double)Coded_Picture_Height - crop - (crop_err - crop)) / (input_aspect/pc_aspect);

	Clip_Top = Clip_Bottom = (int)(crop / 2);
	Squeeze_Height = Coded_Picture_Height - Clip_Top - Clip_Bottom - (((int)height) & ~7);
}

LRESULT CALLBACK ClipResize(HWND hDialog, UINT message, WPARAM wParam, LPARAM lParam)
{
	int i;

	switch (message)
	{
		case WM_INITDIALOG:
			SendDlgItemMessage(hDialog, IDC_FILTMODE, CB_ADDSTRING, 0, (LPARAM)"Bilinear (MMX)");
			SendDlgItemMessage(hDialog, IDC_FILTMODE, CB_ADDSTRING, 0, (LPARAM)"Soft Bicubic (MMX)");
			SendDlgItemMessage(hDialog, IDC_FILTMODE, CB_ADDSTRING, 0, (LPARAM)"Precise Bicubic (MMX)");
			SendDlgItemMessage(hDialog, IDC_FILTMODE, CB_SETCURSEL, Resize_Function, 0);

			SendDlgItemMessage(hDialog, IDC_ASPECT, CB_ADDSTRING, 0, (LPARAM)"Free");
			SendDlgItemMessage(hDialog, IDC_ASPECT, CB_ADDSTRING, 0, (LPARAM)"TV 4:3");
			SendDlgItemMessage(hDialog, IDC_ASPECT, CB_ADDSTRING, 0, (LPARAM)"Cinema 1.85:1");
			SendDlgItemMessage(hDialog, IDC_ASPECT, CB_ADDSTRING, 0, (LPARAM)"Cinema 2.35:1");
			SendDlgItemMessage(hDialog, IDC_ASPECT, CB_SETCURSEL, Aspect_Change, 0);

			SendDlgItemMessage(hDialog, IDC_LEFT_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 32));
			SendDlgItemMessage(hDialog, IDC_RIGHT_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 32));
			SendDlgItemMessage(hDialog, IDC_TOP_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 128));
			SendDlgItemMessage(hDialog, IDC_BOTTOM_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 128));
			SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Width-Clip_Left-Clip_Right)>>1));
			SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Height-Clip_Top-Clip_Bottom)>>3));
			
			UpdateSizes(hDialog);
			ShowWindow(hDialog, SW_SHOW);

			return true;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
			case IDC_FILTMODE:
				{
					Resize_Function = SendDlgItemMessage(hDialog, IDC_FILTMODE, CB_GETCURSEL, 0, 0);
					KillFilters();
					RefreshWindow();
					ShowStatistics(false);
				}
				break;

			case IDC_ASPECT:
				{
					Aspect_Change = SendDlgItemMessage(hDialog, IDC_ASPECT, CB_GETCURSEL, 0, 0);
					AutoSize();
					UpdateSizes(hDialog);
					KillFilters();
					RefreshWindow();
					ShowStatistics(false);
				}
				break;

			case IDC_SIZE_RESET:
				{
					Clip_Top=Clip_Bottom=Clip_Left=Clip_Right=Squeeze_Width=Squeeze_Height=0;
					Aspect_Change = ASPECT_FREE;
					UpdateSizes(hDialog);
					KillFilters();
					RefreshWindow();
					ShowStatistics(false);
				}
				break;

			case IDCANCEL:
				DestroyWindow(hDialog);
				hClipResizeDlg = NULL;
				return true;
			}
			break;

		case WM_HSCROLL:
			switch (GetWindowLong((HWND)lParam, GWL_ID))
			{
				case IDC_LEFT_SLIDER:
					i = SendDlgItemMessage(hDialog, IDC_LEFT_SLIDER, TBM_GETPOS, 0, 0)<<3;
					if (Coded_Picture_Width - i - Clip_Right - Squeeze_Width >= MIN_WIDTH)
					{
						Clip_Left = i;
						sprintf(szTemp, "%d", Clip_Left);
						SetDlgItemText(hDialog, IDC_LEFT, szTemp);

						SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Width-Clip_Left-Clip_Right)>>1));
						SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width)>>1);
						sprintf(szTemp, "%d", Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width);
						SetDlgItemText(hDialog, IDC_WIDTH, szTemp);
					}
					SendDlgItemMessage(hDialog, IDC_LEFT_SLIDER, TBM_SETPOS, 1, Clip_Left>>3);
					AutoSize();
					UpdateSizes(hDialog);
					break;

				case IDC_RIGHT_SLIDER:
					i = SendDlgItemMessage(hDialog, IDC_RIGHT_SLIDER, TBM_GETPOS, 0, 0)<<3;
					if (Coded_Picture_Width - i - Clip_Left - Squeeze_Width >= MIN_WIDTH)
					{
						Clip_Right = i;
						sprintf(szTemp, "%d", Clip_Right);
						SetDlgItemText(hDialog, IDC_RIGHT, szTemp);

						SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Width-Clip_Left-Clip_Right)>>1));
						SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width)>>1);
						sprintf(szTemp, "%d", Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width);
						SetDlgItemText(hDialog, IDC_WIDTH, szTemp);
					}
					SendDlgItemMessage(hDialog, IDC_RIGHT_SLIDER, TBM_SETPOS, 1, Clip_Right>>3);
					AutoSize();
					UpdateSizes(hDialog);
					break;

				case IDC_TOP_SLIDER:
					i = SendDlgItemMessage(hDialog, IDC_TOP_SLIDER, TBM_GETPOS, 0, 0)<<1;
					if (Coded_Picture_Height - i - Clip_Bottom - Squeeze_Height >= MIN_HEIGHT)
					{
						Clip_Top = i;
						sprintf(szTemp, "%d", Clip_Top);
						SetDlgItemText(hDialog, IDC_TOP, szTemp);
		
						SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Height-Clip_Top-Clip_Bottom)>>3));
						SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height)>>3);
						sprintf(szTemp, "%d", Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height);
						SetDlgItemText(hDialog, IDC_HEIGHT, szTemp);
					}
					SendDlgItemMessage(hDialog, IDC_TOP_SLIDER, TBM_SETPOS, 1, Clip_Top>>1);
					break;

				case IDC_BOTTOM_SLIDER:
					i = SendDlgItemMessage(hDialog, IDC_BOTTOM_SLIDER, TBM_GETPOS, 0, 0)<<1;
					if (Coded_Picture_Height - i - Clip_Top - Squeeze_Height >= MIN_HEIGHT)
					{
						Clip_Bottom = i;
						sprintf(szTemp, "%d", Clip_Bottom);
						SetDlgItemText(hDialog, IDC_BOTTOM, szTemp);
						
						SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Height-Clip_Top-Clip_Bottom)>>3));
						SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height)>>3);
						sprintf(szTemp, "%d", Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height);
						SetDlgItemText(hDialog, IDC_HEIGHT, szTemp);
					}
					SendDlgItemMessage(hDialog, IDC_BOTTOM_SLIDER, TBM_SETPOS, 1, Clip_Bottom>>1);
					break;

				case IDC_WIDTH_SLIDER:
					i = Coded_Picture_Width - Clip_Left - Clip_Right - (SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_GETPOS, 0, 0)<<1);
					if (MIN_WIDTH + i + Clip_Left + Clip_Right <= Coded_Picture_Width)
					{
						Squeeze_Width = i;
						sprintf(szTemp, "%d", Coded_Picture_Width - Clip_Left - Clip_Right - Squeeze_Width);
						SetDlgItemText(hDialog, IDC_WIDTH, szTemp);
					}
					SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Width - Clip_Left - Clip_Right - Squeeze_Width)>>1);
					AutoSize();
					UpdateSizes(hDialog);
					break;

				case IDC_HEIGHT_SLIDER:
					i = Coded_Picture_Height - Clip_Top - Clip_Bottom - (SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_GETPOS, 0, 0)<<3);
					if (MIN_HEIGHT + i + Clip_Top + Clip_Bottom <= Coded_Picture_Height)
					{
						Squeeze_Height = i;
						sprintf(szTemp, "%d", Coded_Picture_Height - Clip_Top - Clip_Bottom - Squeeze_Height);
						SetDlgItemText(hDialog, IDC_HEIGHT, szTemp);
					}
					SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height)>>3);
					break;
			}

			KillFilters();
			RefreshWindow();
			ShowStatistics(false);
			break;
	}
    return false;
}

LRESULT CALLBACK Luminance(HWND hDialog, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			SendDlgItemMessage(hDialog, IDC_GAIN_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 256));
			SendDlgItemMessage(hDialog, IDC_GAIN_SLIDER, TBM_SETTICFREQ, 128, 0);
			SendDlgItemMessage(hDialog, IDC_GAIN_SLIDER, TBM_SETPOS, 1, LumGain);
			sprintf(szTemp, "%d", LumGain-128);
			SetDlgItemText(hDialog, IDC_GAIN, szTemp);

			SendDlgItemMessage(hDialog, IDC_OFFSET_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 512));
			SendDlgItemMessage(hDialog, IDC_OFFSET_SLIDER, TBM_SETTICFREQ, 256, 0);
			SendDlgItemMessage(hDialog, IDC_OFFSET_SLIDER, TBM_SETPOS, 1, LumOffset+256);
			sprintf(szTemp, "%d", LumOffset);
			SetDlgItemText(hDialog, IDC_OFFSET, szTemp);

			ShowWindow(hDialog, SW_SHOW);

			return true;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
			case IDC_LUMA_RESET:
				LumGain=128;
				LumOffset=0;
				SendDlgItemMessage(hDialog, IDC_GAIN_SLIDER, TBM_SETPOS, 1, LumGain);
				sprintf(szTemp, "%d", LumGain-128);
				SetDlgItemText(hDialog, IDC_GAIN, szTemp);
				
				SendDlgItemMessage(hDialog, IDC_OFFSET_SLIDER, TBM_SETPOS, 1, LumOffset+256);
				sprintf(szTemp, "%d", LumOffset);
				SetDlgItemText(hDialog, IDC_OFFSET, szTemp);
				RefreshWindow();
				break;
				
			case IDCANCEL:
				DestroyWindow(hDialog);
				hLumDlg = NULL;
				return true;
			}
			break;

		case WM_HSCROLL:
			switch (GetWindowLong((HWND)lParam, GWL_ID))
			{
				case IDC_GAIN_SLIDER:
					LumGain = SendDlgItemMessage(hDialog, IDC_GAIN_SLIDER, TBM_GETPOS, 0, 0);
					sprintf(szTemp, "%d", LumGain-128);
					SetDlgItemText(hDialog, IDC_GAIN, szTemp);	
					break;

				case IDC_OFFSET_SLIDER:
					LumOffset = SendDlgItemMessage(hDialog, IDC_OFFSET_SLIDER, TBM_GETPOS, 0, 0) - 256;
					sprintf(szTemp, "%d", LumOffset);
					SetDlgItemText(hDialog, IDC_OFFSET, szTemp);
					break;
			}

			RefreshWindow();
			break;
	}

    return false;
}

LRESULT CALLBACK Normalization(HWND hDialog, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			SendDlgItemMessage(hDialog, IDC_NORM_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 100));
			SendDlgItemMessage(hDialog, IDC_NORM_SLIDER, TBM_SETTICFREQ, 50, 0);
			SendDlgItemMessage(hDialog, IDC_NORM_SLIDER, TBM_SETPOS, 1, Norm_Ratio);
			sprintf(szTemp, "%d", Norm_Ratio);
			SetDlgItemText(hDialog, IDC_NORM, szTemp);

			ShowWindow(hDialog, SW_SHOW);

			if (Normalization_Flag)
				SendDlgItemMessage(hDialog, IDC_NORM_CHECK, BM_SETCHECK, BST_CHECKED, 0);
			return true;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDC_NORM_CHECK:
					if (SendDlgItemMessage(hDialog, IDC_NORM_CHECK, BM_GETCHECK, 1, 0)==BST_CHECKED)
					{
						CheckMenuItem(hMenu, IDM_NORM, MF_CHECKED);
						Normalization_Flag = true;
					}
					else
					{
						CheckMenuItem(hMenu, IDM_NORM, MF_UNCHECKED);
						Normalization_Flag = false;
					}
					break;

				case IDCANCEL:
					DestroyWindow(hDialog);
					hNormDlg = NULL;
					return true;
			}
			break;

		case WM_HSCROLL:
			if (GetWindowLong((HWND)lParam, GWL_ID)==IDC_NORM_SLIDER)
			{
				Norm_Ratio = SendDlgItemMessage(hDialog, IDC_NORM_SLIDER, TBM_GETPOS, 0, 0);
				sprintf(szTemp, "%d", Norm_Ratio);
				SetDlgItemText(hDialog, IDC_NORM, szTemp);
			}
			break;
	}
    return false;
}

/* register the window class */
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize			= sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= false;
	wcex.cbWndExtra		= false;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_MOVIE);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= CreateSolidBrush(MASKCOLOR);
	wcex.lpszMenuName	= (LPCSTR)IDC_GUI;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_MOVIE);

	return RegisterClassEx(&wcex);
}

char *ExtFilter[] = 
{
	".avi", ".d2s", ".d2v", ".wav"
};

BOOL PopFileDlg(PTSTR pstrFileName, HWND hOwner, int Status)
{
	static char *szFilter, *ext;	
	int count = 0;

	switch (Status)
	{
		case OPEN_VOB:
			szFilter = TEXT ("MPEG-2 Program Stream (*.vob; *.m2p; *.m2v; *.mpv)\0*.vob;*.m2p;*.m2v;*.mpv\0")  \
				TEXT ("MPEG-2 Transport Stream (*.trp; *.ts)\0*.trp;*.ts\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;

		case SAVE_AVI:
			szFilter = TEXT ("AVI File (*.avi)\0*.avi; *.ac3; *.wav; *.mpa\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;

		case OPEN_D2V:
			szFilter = TEXT ("DVD2AVI Project File (*.d2v)\0*.d2v\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;

		case SAVE_D2V:
			szFilter = TEXT ("DVD2AVI Project File (*.d2v)\0*.d2v\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;
		
		case OPEN_SESS:
			szFilter = TEXT ("DVD2AVI Session File (*.d2s)\0*.d2s\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;

		case SAVE_SESS:
			szFilter = TEXT ("DVD2AVI Session File (*.d2s)\0*.d2s\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;

		case OPEN_WAV:
		case SAVE_WAV:
			szFilter = TEXT ("WAV File (*.wav)\0*.wav\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;
	}

	ofn.lStructSize       = sizeof (OPENFILENAME) ;
	ofn.hwndOwner         = hOwner ;
	ofn.hInstance         = hInst ;
	ofn.lpstrFilter       = szFilter ;
	ofn.nMaxFile          = _MAX_PATH ;
	ofn.nMaxFileTitle     = _MAX_PATH ;
	ofn.lpstrFile         = pstrFileName ;

	switch (Status)
	{
		case OPEN_VOB:
		case OPEN_D2V:
		case OPEN_SESS:
		case OPEN_WAV:
			ofn.Flags = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST;
			return GetOpenFileName(&ofn);

		case SAVE_WAV:
			count++;
		case SAVE_D2V:
			count++;
		case SAVE_SESS:
			count++;
		case SAVE_AVI:
			ofn.Flags = OFN_HIDEREADONLY | OFN_ALLOWMULTISELECT | OFN_EXPLORER;
			if (GetSaveFileName(&ofn))
			{
				ext = strrchr(pstrFileName, '.');

				if (ext!=NULL && !_strnicmp(ext, ExtFilter[count], 4))
					strncpy(ext, ext+4, 1);

				return true;
			}
	}

	return false;
}

static void ShowStatistics(bool refresh)
{
	if (refresh)
	{
		if (Statistics_Flag)
			DestroyWindow(hDlg);

		Statistics_Flag = true;
		hDlg = CreateDialog(hInst, (LPCTSTR)IDD_STATISTICS, hWnd, (DLGPROC)Statistics);
	}

	if (Statistics_Flag)
	{
		GetWindowRect(hDlg, &crect);
		GetWindowRect(hWnd, &wrect);
		MoveWindow(hDlg, wrect.right, wrect.top+Edge_Height-Edge_Width/2, crect.right-crect.left, crect.bottom-crect.top, true);
		ShowWindow(hDlg, SW_SHOW);
	}
}

static void DisableControl(int audio)
{
	DragAcceptFiles(hWnd, false);

	EnableWindow(hTrack, false);
	EnableWindow(hLeftButton, false);
	EnableWindow(hLeftArrow, false);
	EnableWindow(hRightArrow, false);
	EnableWindow(hRightButton, false);

	EnableMenuItem(hMenu, 1, MF_BYPOSITION | MF_GRAYED);
	EnableMenuItem(hMenu, 2, MF_BYPOSITION | MF_GRAYED);
	DrawMenuBar(hWnd);

	if (hClipResizeDlg!=NULL)
	{
		DestroyWindow(hClipResizeDlg);
		hClipResizeDlg = NULL;
	}

	if (hLumDlg!=NULL)
	{
		DestroyWindow(hLumDlg);
		hLumDlg = NULL;
	}

	if (hNormDlg!=NULL)
	{
		DestroyWindow(hNormDlg);
		hNormDlg = NULL;
	}
}

static void ClearTrack()
{
	CheckMenuItem(hMenu, IDM_TRACK_NONE, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_1, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_2, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_3, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_4, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_5, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_6, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_7, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_8, MF_UNCHECKED);
}

static void CheckINI()
{
	
	__asm
	{
		mov			eax, 1
		cpuid
		test		edx, 0x00800000		// STD MMX
		jz			TEST_SSE
		mov			[cpu.mmx], 1
TEST_SSE:
		test		edx, 0x02000000		// STD SSE
		jz			TEST_3DNOW
		mov			[cpu.ssemmx], 1
		mov			[cpu.ssefpu], 1

		test		edx, 0x04000000	// STD SSE2	TRB 12/30/2001
		jz			TEST_3DNOW
		mov			[cpu.sse2mmx], 1

TEST_3DNOW:
		mov			eax, 0x80000001
		cpuid
		test		edx, 0x80000000		// 3D NOW
		jz			TEST_SSEMMX
		mov			[cpu._3dnow], 1
TEST_SSEMMX:
		test		edx, 0x00400000		// SSE MMX
		jz			TEST_END
		mov			[cpu.ssemmx], 1
TEST_END:
	}
	
	if (cpu.mmx)
		CheckMenuItem(hMenu, IDM_MMX, MF_CHECKED);

	if (cpu.sse2mmx)
	{
		CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_CHECKED);
		DeleteMenu(hMenu, IDM_IDCT_SSEMMX, 0);
		DeleteMenu(hMenu, IDM_IDCT_MMX, 0);
	}
	else if (cpu.ssemmx)
	{
		CheckMenuItem(hMenu, IDM_SSEMMX, MF_CHECKED);
		DeleteMenu(hMenu, IDM_IDCT_SSE2MMX, 0);
		DeleteMenu(hMenu, IDM_IDCT_MMX, 0);
	}
	else
	{
		DeleteMenu(hMenu, IDM_IDCT_SSEMMX, 0);
		DeleteMenu(hMenu, IDM_IDCT_SSE2MMX, 0);
	}
	if (cpu._3dnow)
		CheckMenuItem(hMenu, IDM_3DNOW, MF_CHECKED);

	if (cpu.ssefpu)
		CheckMenuItem(hMenu, IDM_SSEFPU, MF_CHECKED);

	if ((INIFile = fopen("DVD2AVI.ini", "r")) == NULL)
	{
NEW_VERSION:
		INIT_X = INIT_Y = 100;
		INI_Version = INI_VERSION;

		if (cpu.sse2mmx)
			iDCT_Flag = IDCT_SSE2MMX;
		else if (cpu.ssemmx)
			iDCT_Flag = IDCT_SSEMMX;
		else
			iDCT_Flag = IDCT_MMX;

		Scale_Flag = true;
		FO_Flag = FO_NONE;
		Track_Flag = TRACK_NONE;
		Format_Flag = FORMAT_AUTO;
		AC3_Flag = AUDIO_DECODE;
		DRC_Flag = DRC_NORMAL;
		DSDown_Flag = false;
		MPA_Flag = AUDIO_DEMUXONE;
		SRC_Flag = SRC_NONE;
		Norm_Ratio = 100;
		Priority_Flag = PRIORITY_NORMAL;
		Resize_Function = FILTER_BILINEAR;
		Store_Flag = STORE_YUY2;
		MPEG2_Transport_VideoPID = 0x0011; // arbitrary init value
		MPEG2_Transport_AudioPID = 0x0014; // arbitrary init value
	}
	else
	{
		fscanf(INIFile, "INI_Version=%d\n", &INI_Version);
		if (INI_Version != INI_VERSION)
		{
			fclose(INIFile);
			goto NEW_VERSION;
		}
		
		fscanf(INIFile, "Window_Position=%d,%d\n", &INIT_X, &INIT_Y);
		if (INIT_X < 0 || INIT_X + 100 > GetSystemMetrics(SM_CXSCREEN) || 
			INIT_Y < 0 || INIT_Y + 100 > GetSystemMetrics(SM_CYSCREEN))
			INIT_X = INIT_Y = 100;
		
		fscanf(INIFile, "iDCT_Algorithm=%d\n", &iDCT_Flag);
		fscanf(INIFile, "YUVRGB_Scale=%d\n", &Scale_Flag);
		fscanf(INIFile, "Field_Operation=%d\n", &FO_Flag);
		fscanf(INIFile, "Track_Number=%d\n", &Track_Flag);
		fscanf(INIFile, "Channel_Format=%d\n", &Format_Flag);
		fscanf(INIFile, "AC3=%d\n", &AC3_Flag);
		fscanf(INIFile, "DR_Control=%d\n", &DRC_Flag);
		fscanf(INIFile, "DS_Downmix=%d\n", &DSDown_Flag);
		fscanf(INIFile, "MPA=%d\n", &MPA_Flag);
		fscanf(INIFile, "SRC_Precision=%d\n", &SRC_Flag);
		fscanf(INIFile, "Norm_Ratio=%d\n", &Norm_Ratio);
		fscanf(INIFile, "MPEG2_Transport_PID=%X,%X\n", &MPEG2_Transport_VideoPID,
			&MPEG2_Transport_AudioPID );

		if (Norm_Ratio > 100)
		{
			Norm_Ratio -= 100;
			Normalization_Flag = true;
		}
		else
			Normalization_Flag = false;
		fscanf(INIFile, "Process_Priority=%d\n", &Priority_Flag);
		fscanf(INIFile, "Store_Flag=%d\n", &Store_Flag);
		
		fclose(INIFile);
	}
}

static void CheckUncheckMenus()
{
	CheckMenuItem(hMenu, IDM_KEY_OFF, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_KEY_INPUT, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_KEY_OP, MF_UNCHECKED);
	
	if (!PlugIn_Flag)
		KeyOp_Flag = KEY_OFF;
	
	switch (KeyOp_Flag)
	{
	case KEY_OFF:
		CheckMenuItem(hMenu, IDM_KEY_OFF, MF_CHECKED);
		break;
		
	case KEY_INPUT:
		CheckMenuItem(hMenu, IDM_KEY_INPUT, MF_CHECKED);
		break;
		
	case KEY_OP:
		CheckMenuItem(hMenu, IDM_KEY_OP, MF_CHECKED);
		break;
	}
	
	CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_IDCT_REF, MF_UNCHECKED);
	
	if (cpu.ssemmx && iDCT_Flag==IDCT_MMX)
		iDCT_Flag = IDCT_SSEMMX;

	if (cpu.ssemmx && iDCT_Flag==IDCT_SSE2MMX)
		iDCT_Flag = IDCT_SSE2MMX;
	
	if (!cpu.ssemmx && iDCT_Flag==IDCT_SSEMMX)
		iDCT_Flag = IDCT_MMX;
	
	switch (iDCT_Flag)
	{
	case IDCT_MMX:
		CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_CHECKED);
		break;
		
	case IDCT_SSEMMX:
		CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_CHECKED);
		break;

	case IDCT_SSE2MMX:
		CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_CHECKED);
		break;
		
	case IDCT_FPU:
		CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_CHECKED);
		break;
		
	case IDCT_REF:
		CheckMenuItem(hMenu, IDM_IDCT_REF, MF_CHECKED);
		break;
	}
	
	if (Store_Flag == STORE_YUY2)
	{
		CheckMenuItem(hMenu, IDM_STORE_RGB24, MF_UNCHECKED);
		CheckMenuItem(hMenu, IDM_STORE_YUY2, MF_CHECKED);
		EnableMenuItem(hMenu, IDM_BMP, MF_GRAYED);
	}
	else
	{
		CheckMenuItem(hMenu, IDM_STORE_RGB24, MF_CHECKED);
		CheckMenuItem(hMenu, IDM_STORE_YUY2, MF_UNCHECKED);
		EnableMenuItem(hMenu, IDM_BMP, MF_ENABLED);
	}

	CheckMenuItem(hMenu, IDM_PCSCALE, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TVSCALE, MF_UNCHECKED);
	
	if (Scale_Flag)
	{
		RGB_Scale = 0x1000254310002543;
		RGB_Offset = 0x0010001000100010;
		RGB_CBU = 0x0000408D0000408D;
		RGB_CGX = 0xF377E5FCF377E5FC;
		RGB_CRV = 0x0000331300003313;
		
		CheckMenuItem(hMenu, IDM_PCSCALE, MF_CHECKED);
	}
	else
	{
		RGB_Scale = 0x1000200010002000;
		RGB_Offset = 0x0000000000000000;
		RGB_CBU = 0x000038B4000038B4;
		RGB_CGX = 0xF4FDE926F4FDE926;
		RGB_CRV = 0x00002CDD00002CDD;
		
		CheckMenuItem(hMenu, IDM_TVSCALE, MF_CHECKED);
	}
	
	CheckMenuItem(hMenu, IDM_FO_NONE, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_FO_FILM, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_FO_SWAP, MF_UNCHECKED);
	
	switch (FO_Flag)
	{
	case FO_NONE:
		CheckMenuItem(hMenu, IDM_FO_NONE, MF_CHECKED);
		break;
		
	case FO_FILM:
		CheckMenuItem(hMenu, IDM_FO_FILM, MF_CHECKED);
		break;
		
	case FO_SWAP:
		CheckMenuItem(hMenu, IDM_FO_SWAP, MF_CHECKED);
		break;
	}
	
	CheckMenuItem(hMenu, IDM_TRACK_NONE, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_1, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_2, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_3, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_4, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_5, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_6, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_7, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_8, MF_UNCHECKED);
	
	switch (Track_Flag)
	{
	case TRACK_NONE:
		CheckMenuItem(hMenu, IDM_TRACK_NONE, MF_CHECKED);
		break;
		
	case TRACK_1:
		CheckMenuItem(hMenu, IDM_TRACK_1, MF_CHECKED);
		break;
		
	case TRACK_2:
		CheckMenuItem(hMenu, IDM_TRACK_2, MF_CHECKED);
		break;
		
	case TRACK_3:
		CheckMenuItem(hMenu, IDM_TRACK_3, MF_CHECKED);
		break;
		
	case TRACK_4:
		CheckMenuItem(hMenu, IDM_TRACK_4, MF_CHECKED);
		break;
		
	case TRACK_5:
		CheckMenuItem(hMenu, IDM_TRACK_5, MF_CHECKED);
		break;
		
	case TRACK_6:
		CheckMenuItem(hMenu, IDM_TRACK_6, MF_CHECKED);
		break;
		
	case TRACK_7:
		CheckMenuItem(hMenu, IDM_TRACK_7, MF_CHECKED);
		break;
		
	case TRACK_8:
		CheckMenuItem(hMenu, IDM_TRACK_8, MF_CHECKED);
		break;
	}
	
	CheckMenuItem(hMenu, IDM_AC3, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_MPA, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_LPCM, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_SELECT, MF_UNCHECKED);
	
	switch (Format_Flag)
	{
	case FORMAT_AC3:
		CheckMenuItem(hMenu, IDM_AC3, MF_CHECKED);
		break;
		
	case FORMAT_MPA:
		CheckMenuItem(hMenu, IDM_MPA, MF_CHECKED);
		break;
		
	case FORMAT_LPCM:
		CheckMenuItem(hMenu, IDM_LPCM, MF_CHECKED);
		break;
		
	case FORMAT_AUTO:
		CheckMenuItem(hMenu, IDM_SELECT, MF_CHECKED);
		break;
	}
	
	CheckMenuItem(hMenu, IDM_AC3_DEMUXALL, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_AC3_DEMUXONE, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_AC3_DECODE, MF_UNCHECKED);
	
	switch (AC3_Flag)
	{
	case AUDIO_DEMUXALL:
		CheckMenuItem(hMenu, IDM_AC3_DEMUXALL, MF_CHECKED);
		break;
		
	case AUDIO_DEMUXONE:
		CheckMenuItem(hMenu, IDM_AC3_DEMUXONE, MF_CHECKED);
		break;
		
	case AUDIO_DECODE:
		CheckMenuItem(hMenu, IDM_AC3_DECODE, MF_CHECKED);
		break;
	}
	
	CheckMenuItem(hMenu, IDM_DRC_NONE, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_DRC_LIGHT, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_DRC_NORMAL, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_DRC_HEAVY, MF_UNCHECKED);			
	
	switch (DRC_Flag)
	{
	case DRC_NONE:
		CheckMenuItem(hMenu, IDM_DRC_NONE, MF_CHECKED);
		break;
		
	case DRC_LIGHT:
		CheckMenuItem(hMenu, IDM_DRC_LIGHT, MF_CHECKED);
		break;
		
	case DRC_NORMAL:
		CheckMenuItem(hMenu, IDM_DRC_NORMAL, MF_CHECKED);
		break;
		
	case DRC_HEAVY:
		CheckMenuItem(hMenu, IDM_DRC_HEAVY, MF_CHECKED);
		break;
	}
	
	if (DSDown_Flag)
		CheckMenuItem(hMenu, IDM_DSDOWN, MF_CHECKED);
	else
		CheckMenuItem(hMenu, IDM_DSDOWN, MF_UNCHECKED);
	
	CheckMenuItem(hMenu, IDM_MPA_DEMUXALL, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_MPA_DEMUXONE, MF_UNCHECKED);
	
	switch (MPA_Flag)
	{
	case AUDIO_DEMUXALL:
		CheckMenuItem(hMenu, IDM_MPA_DEMUXALL, MF_CHECKED);
		break;
		
	case AUDIO_DEMUXONE:
		CheckMenuItem(hMenu, IDM_MPA_DEMUXONE, MF_CHECKED);
		break;
	}
	
	CheckMenuItem(hMenu, IDM_SRC_NONE, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_SRC_LOW, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_SRC_MID, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_UNCHECKED);
	
	switch (SRC_Flag)
	{
	case SRC_NONE:
		CheckMenuItem(hMenu, IDM_SRC_NONE, MF_CHECKED);
		break;
		
	case SRC_LOW:
		CheckMenuItem(hMenu, IDM_SRC_LOW, MF_CHECKED);
		break;
		
	case SRC_MID:
		CheckMenuItem(hMenu, IDM_SRC_MID, MF_CHECKED);
		break;
		
	case SRC_HIGH:
		CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_CHECKED);
		break;
		
	case SRC_UHIGH:
		CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_CHECKED);
		break;
	}
	
	if (Normalization_Flag)
		CheckMenuItem(hMenu, IDM_NORM, MF_CHECKED);
	else
		CheckMenuItem(hMenu, IDM_NORM, MF_UNCHECKED);
	
	CheckMenuItem(hMenu, IDM_PP_HIGH, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_PP_NORMAL, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_PP_LOW, MF_UNCHECKED);
	
	switch (Priority_Flag)
	{
	case PRIORITY_HIGH:
		CheckMenuItem(hMenu, IDM_PP_HIGH, MF_CHECKED);
		SetPriorityClass(hProcess, HIGH_PRIORITY_CLASS);
		break;
		
	case PRIORITY_NORMAL:
		CheckMenuItem(hMenu, IDM_PP_NORMAL, MF_CHECKED);
		break;
		
	case PRIORITY_LOW:
		CheckMenuItem(hMenu, IDM_PP_LOW, MF_CHECKED);
		SetPriorityClass(hProcess, IDLE_PRIORITY_CLASS);
		break;
	}
}

static void Recovery()
{
	int i;

	if (Check_Flag)
	{
		if (DDOverlay_Avail)
		{
			DDOverlay_Avail = false;
			CheckMenuItem(hMenu, IDM_DIRECTDRAW, MF_UNCHECKED);
			IDirectDrawSurface_UpdateOverlay(lpOverlay, NULL, lpPrimary, NULL, DDOVER_HIDE, NULL);
		}

		if (lpDD2)
		{
			lpDD2->Release();
			lpDD2 = 0;
		}
		if (lpDD)
		{
			lpDD->Release();
			lpDD = 0;
		}

		for (i=0; i<3; i++)
		{
			aligned_free(backward_reference_frame[i]);
			aligned_free(forward_reference_frame[i]);
			aligned_free(auxframe[i]);
		}

		aligned_free(u422);
		aligned_free(v422);
		aligned_free(u444);
		aligned_free(v444);
		aligned_free(rgb24);
		aligned_free(rgb24_temp);
		aligned_free(rgb24_temp2);
		aligned_free(yuy2);
		aligned_free(yuy2_temp);
		aligned_free(lum);
	}

	Check_Flag = false;

	SendMessage(hTrack, TBM_SETPOS, (WPARAM) true, 0);
	SendMessage(hTrack, TBM_SETSEL, (WPARAM) true, (LPARAM) MAKELONG(0, 0));

	DisableControl(false);

	EnableMenuItem(hMenu, IDM_SAVE, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_SAVE_D2V, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_SAVE_SESS, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_BMP, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_PROCESS_WAV, MF_ENABLED);

	LumGain = 128; LumOffset = 0;

	Clip_Left = Clip_Right = Clip_Top = Clip_Bottom = Squeeze_Width = Squeeze_Height = 0;
	Aspect_Change = ASPECT_UNKNOWN;

	PreScale_Ratio = 1.0;
	CheckMenuItem(hMenu, IDM_PRESCALE, MF_UNCHECKED);

	SetWindowText(hWnd, "DVD2AVI");

	if (File_Limit)
	{
		EnableMenuItem(hMenu, IDM_PROCESS_WAV, MF_GRAYED);

		ZeroMemory(&process, sizeof(PROCESS));
		process.trackright = TRACK_PITCH;

		SystemStream_Flag = false;
		Display_Flag = true;

		for (i=0; i<File_Limit; i++)
		{
			process.length[i] = _filelengthi64(Infile[i]);
			process.total += process.length[i];
		}
	}
	else
		DragAcceptFiles(hWnd, true);

	ResizeWindow(INIT_WIDTH, INIT_HEIGHT);
}

void ResizeWindow(int width, int height)
{
	MoveWindow(hTrack, 0, height, width-4*TRACK_HEIGHT, TRACK_HEIGHT, true);
	MoveWindow(hLeftButton, width-4*TRACK_HEIGHT, height, TRACK_HEIGHT, TRACK_HEIGHT, true);
	MoveWindow(hLeftArrow, width-3*TRACK_HEIGHT, height, TRACK_HEIGHT, TRACK_HEIGHT, true);
	MoveWindow(hRightArrow, width-2*TRACK_HEIGHT, height, TRACK_HEIGHT, TRACK_HEIGHT, true);
	MoveWindow(hRightButton, width-TRACK_HEIGHT, height, TRACK_HEIGHT, TRACK_HEIGHT, true);

	GetWindowRect(hWnd, &wrect);
	GetClientRect(hWnd, &crect);
	Edge_Width = wrect.right - wrect.left - crect.right;
	Edge_Height = wrect.bottom - wrect.top - crect.bottom;

	MoveWindow(hWnd, wrect.left, wrect.top, width+Edge_Width, height+Edge_Height+TRACK_HEIGHT, true);
}

void InitOverlayIfAvailable()
{
	if (!DDOverlay_Avail)
	{
		if (DirectDrawCreate(NULL, &lpDD, NULL)==DD_OK)
		{
			if (lpDD->QueryInterface(IID_IDirectDraw2, (LPVOID*)&lpDD2)==DD_OK)
			{
				if (lpDD2->SetCooperativeLevel(hWnd, DDSCL_NORMAL)==DD_OK)
				{
					ZeroMemory(&ddsd, sizeof(DDSURFACEDESC));
					ddsd.dwSize = sizeof(DDSURFACEDESC);
					ddsd.dwFlags = DDSD_CAPS;
					ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_VIDEOMEMORY;
					
					if (lpDD2->CreateSurface(&ddsd, &lpPrimary, NULL)==DD_OK)
					{
						ZeroMemory(&halcaps, sizeof(DDCAPS));
						halcaps.dwSize = sizeof(DDCAPS);
						
						if (lpDD2->GetCaps(&halcaps, NULL)==DD_OK)
						{
							if (halcaps.dwCaps & DDCAPS_OVERLAY)
							{
								DDPIXELFORMAT ddPixelFormat;
								ddPixelFormat.dwFlags = DDPF_FOURCC;
								ddPixelFormat.dwFourCC = mmioFOURCC('Y','U','Y','2');
								ddPixelFormat.dwYUVBitCount = 16;
								
								ZeroMemory(&ddsd, sizeof(DDSURFACEDESC));
								ddsd.dwSize = sizeof(DDSURFACEDESC);
								ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT;
								ddsd.ddsCaps.dwCaps = DDSCAPS_OVERLAY | DDSCAPS_VIDEOMEMORY;
								ddsd.dwWidth = Coded_Picture_Width;
								ddsd.dwHeight = Coded_Picture_Height;
								
								memcpy(&(ddsd.ddpfPixelFormat), &ddPixelFormat, sizeof(DDPIXELFORMAT));
								int i = lpDD2->CreateSurface(&ddsd, &lpOverlay, NULL);
								if (i==DD_OK)
								{
									DDOverlay_Avail = true;
									CheckMenuItem(hMenu, IDM_DIRECTDRAW, MF_CHECKED);
									
									ZeroMemory(&ddofx, sizeof(DDOVERLAYFX));
									ddofx.dwSize = sizeof(DDOVERLAYFX);
									
									ddofx.dckDestColorkey.dwColorSpaceLowValue = DDColorMatch(lpPrimary, MASKCOLOR);
									ddofx.dckDestColorkey.dwColorSpaceHighValue = ddofx.dckDestColorkey.dwColorSpaceLowValue;

									// Initialize to black
									if (IDirectDrawSurface_Lock(lpOverlay, NULL, &ddsd, 0, NULL)==DD_OK)
									{
										int i,j;
										unsigned char* dst = (unsigned char *)ddsd.lpSurface;
										
										for (i=0; i<Coded_Picture_Height; i++)
										{
											unsigned char *old = dst;
											for (j=0; j<Coded_Picture_Width; j++)
											{
												*dst++ = 0;
												*dst++ = 128;
											}
											dst = old + ddsd.lPitch;
										}
										
										IDirectDrawSurface_Unlock(lpOverlay, NULL);
									}
								}
							}
						}
					}
				}
			}
		}
		if (!DDOverlay_Avail && !AutoQuit_Flag)
			MessageBox(0, "You video card cannot handle YUV display. Conversion will work, but you will not get any display in YUV mode", "Warning", MB_OK|MB_ICONEXCLAMATION);
	}
}

static DWORD DDColorMatch(LPDIRECTDRAWSURFACE pdds, COLORREF rgb)
{
	COLORREF	rgbT;
	DWORD		dw = CLR_INVALID;
	HRESULT		hres;
	HDC			hdc;

	if (IDirectDrawSurface_GetDC(pdds, &hdc)==DD_OK)
	{
		rgbT = GetPixel(hdc, 0, 0);
		SetPixel(hdc, 0, 0, rgb);
		IDirectDrawSurface_ReleaseDC(pdds, hdc);
	}

	ZeroMemory(&ddsd, sizeof(DDSURFACEDESC));
	ddsd.dwSize = sizeof(DDSURFACEDESC);

	while ((hres = IDirectDrawSurface_Lock(pdds, NULL, &ddsd, 0, NULL)) == DDERR_WASSTILLDRAWING)
		;

	if (hres==DD_OK)
	{
		dw = *(DWORD *) ddsd.lpSurface;
		if (ddsd.ddpfPixelFormat.dwRGBBitCount < 32)
			dw &= (1 << ddsd.ddpfPixelFormat.dwRGBBitCount) - 1;
		IDirectDrawSurface_Unlock(pdds, NULL);
	}

	if (IDirectDrawSurface_GetDC(pdds, &hdc)==DD_OK)
	{
		SetPixel(hdc, 0, 0, rgbT);
		IDirectDrawSurface_ReleaseDC(pdds, hdc);
	}

	return dw;
}

static void RefreshWindow()
{
	if (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
	{
		Fault_Flag = false; Display_Flag = true;
		Write_Frame(backward_reference_frame, d2v_backward, 0);
	}
}

